c **********************************************************************
c *   Input/Output of the simulated distributions                      *
c *                                                                    *
c *   Input variables:                                                 *
c *     f_io          : Portion of the file name relevant to Run no.   *
c *                                                                    *
c *   Output variables:                                                *
c *     nf3d          : Number of I/O devices used for 3-D data        *
c *                   : Device no. is equal to or larger than 20.      *
c *     nf2d          : Number of I/O devices used for 2-D data        *
c *                   : Device no. is equal to or larger than 50.      *
c **********************************************************************
c
      block data io_file_name
c
      parameter ( mf3d=15, mf2d=4 )
      character (len=4) :: f_3d, f_2d
      common  /io_name/ f_3d(mf3d), f_2d(mf2d)
c
      data  f_3d / 'u___', 'v___', 'w___', 't___', 'q___',
     :             'fub_', 'fvb_', 'fwb_', 'ftb_', 'fqb_',
     :             'wsb_', 'pd__', 'ql__', 'fm__', 'fh__'/
      data  f_2d / 'ts__', 'flm_', 'flt_', 'flq_'/
c
      end
c **********************************************************************
c
      subroutine  data_open ( f_io, nf3d, nf2d )
c
      character (len=5) :: f_io
c
      parameter ( mf3d=15, mf2d=4 )
      character (len=4) :: f_3d, f_2d
      common  /io_name/ f_3d(mf3d), f_2d(mf2d)
c
c  ***  For the initial distributions, nf3d = 5 and nf2d = 0  ***
        nf3d = mf3d
        nf2d = mf2d
      if ( f_io(4:5) .eq. '00' ) then
        nf3d = 5
        nf2d = 0
      end if
c
      do i = 1,nf3d
        open ( i+19,file=f_3d(i)//f_io,form='unformatted' )
      end do
      do i = 1,nf2d
        open ( i+49,file=f_2d(i)//f_io,form='unformatted' )
      end do
c
      return
      end
c **********************************************************************
c
      subroutine  data_close ( nf3d, nf2d )
c
      do i = 1,nf3d
        close ( i+19 )
      end do
      do i = 1,nf2d
        close ( i+49 )
      end do
c
      return
      end
c **********************************************************************
c
      subroutine  data_output ( nf3d, nf2d )
c
      include  "Declare.h"
c
      parameter ( mf3d=15, mf2d=4 )
      character (len=4) :: f_3d, f_2d
      common  /io_name/ f_3d(mf3d), f_2d(mf2d)
c
      do i = 1,nf3d
        mt = i +19
c
        select case ( f_3d(i) )
          case ( 'u___' );  write(mt)  u
          case ( 'v___' );  write(mt)  v
          case ( 'w___' );  write(mt)  w
          case ( 't___' );  write(mt)  t
          case ( 'q___' );  write(mt)  q
          case ( 'fub_' );  write(mt)  fub
          case ( 'fvb_' );  write(mt)  fvb
          case ( 'fwb_' );  write(mt)  fwb
          case ( 'ftb_' );  write(mt)  ftb
          case ( 'fqb_' );  write(mt)  fqb
          case ( 'wsb_' );  write(mt)  wsb
          case ( 'pd__' );  write(mt)  pd
          case ( 'ql__' );  write(mt)  ql
          case ( 'fm__' );  write(mt)  fm
          case ( 'fh__' );  write(mt)  fh
          case default
        end select
c
      end do
c
      do i = 1,nf2d
        mt = i +49
c
        select case ( f_2d(i) )
          case ( 'ts__' );  write(mt)  ts
          case ( 'flm_' );  write(mt)  flm
          case ( 'flt_' );  write(mt)  flt
          case ( 'flq_' );  write(mt)  flq
          case default
        end select
c
      end do
c
      return
      end
c **********************************************************************
c
      subroutine  data_restart ( f_io )
c
      include  "Declare.h"
c
      character (len=5) :: f_io
c
      parameter ( mf3d=15, mf2d=4 )
      character (len=4) :: f_3d, f_2d
      common  /io_name/ f_3d(mf3d), f_2d(mf2d)
c
      call data_open ( f_io, nf3d, nf2d )
c
      do i = 1,nf3d
        mt = i +19
c
      do j = 1,999
        select case ( f_3d(i) )
          case ( 'u___' );  read (mt,end=100)  u
          case ( 'v___' );  read (mt,end=100)  v
          case ( 'w___' );  read (mt,end=100)  w
          case ( 't___' );  read (mt,end=100)  t
          case ( 'q___' );  read (mt,end=100)  q
          case ( 'fub_' );  read (mt,end=100)  fub
          case ( 'fvb_' );  read (mt,end=100)  fvb
          case ( 'fwb_' );  read (mt,end=100)  fwb
          case ( 'ftb_' );  read (mt,end=100)  ftb
          case ( 'fqb_' );  read (mt,end=100)  fqb
          case ( 'wsb_' );  read (mt,end=100)  wsb
          case ( 'pd__' );  read (mt,end=100)  pd
          case ( 'ql__' );  read (mt,end=100)  ql
          case default;     goto 100
        end select
      end do
c
  100 continue
      end do
c
      do i = 1,nf2d
        mt = i +49
c
      do j = 1,999
        select case ( f_2d(i) )
          case ( 'ts__' );  read (mt,end=200)  ts
          case default;     goto 200
        end select
      end do
c
  200 continue
      end do
c
      call data_close ( nf3d, nf2d )
c
      do j = 1,ny
      do k = 1,mz
      do i = 1,mx
        vt (i,k,j) = 0.0
        vq (i,k,j) = 0.0
        cld(i,k,j) = 0.0
        rhr(i,k,j) = 0.0
        rfx(i,k,j) = 0.0
      end do
      end do
      end do
c
      return
      end
