c **********************************************************************
c *   A large-eddy simulation (LES) model for the Earth Simulator      *
c *                                                                    *
c *                                   Apr/2005  M. Nakanishi (N.D.A)   *
c *                                             naka@nda.ac.jp         *
c **********************************************************************
c
      include  "Declare.h"
c
      call arrangement (
     :                  nrun, nstep,  jout,  lout,
     :                  isgs,  isfc,  irad,  icon,
     :                  jsec,  rlat,  elat,
     :                   z0m,   z0h,  tfmx,  qfmx, dhour,
     :                 tdeep,  swet,  crso,  fkso,
     :                  albd,  clov,  ulov,
     :                  nspl,  nspu,   csp,
     :                  nf3d,  nf2d )
c
      fac1 = 1.0
      pmx  = 0.0
      pmy  = 0.0
c
        aa  = 1.0
        new = 0
      if ( nrun .lt. 0 ) then
        aa  = 1.5
        new = 1
c
c  ***  Download of ust for the restart mode  ***
        ust = fub(1,nz+1,1)
      end if
c
                                           itmp = 4
                                           ihum = 5
      if ( irad .eq. 0 .and. icon .le. 0 ) itmp = 0
      if (                   icon .le. 0 ) ihum = 0
c
      if ( irad .ne. 0                   ) irad = 60.0/dt +0.5
c
c  ***********************  Simulation start  ***********************
      do istep = 1,nstep
c
        chour = ( jsec + dt*(istep-1) )/3600.0
c
c  ***  Physical processes  ***
        call hydro_pressure
c
        if     ( icon              .gt. 0 )
     :    call condensation
        if     ( irad              .gt. 0 ) then
          if   ( mod(istep-1,irad) .eq. 0 )
     :    call radiation ( chour, rlat, elat, albd, clov, ulov )
        end if
c
        if     ( isfc              .eq. 0 ) then
          call surface_auto      (
     :                new, z0m, z0h, tdeep, swet, crso, fkso,
     :                ust, duz )
        elseif ( isfc              .gt. 0 ) then
          call surface_prescript ( isfc,
     :                new, z0m, z0h, tfmx, qfmx, dhour, chour,
     :                ust, duz )
        else
          call surface_stress_free
        end if
c
        call sgs_model ( isgs, isfc, ust, duz )
c
c  ***  Temperature & Humidity  ***
          call flux_scalar    ( t, fh, flt, fac1, tnu )
          call physical_term  ( itmp )
          call predict_scalar ( aa, t, ftb, gamt )
c
        if ( icon .ge. 0 ) then
          call flux_scalar    ( q, fh, flq, fac1, tnu )
          call physical_term  ( ihum )
          call predict_scalar ( aa, q, fqb, gamq )
        end if
c
c  ***  Momentum  ***
        call flux_moment
        call hydro_subtraction
        call predict_moment ( aa, pmx, pmy )
c
        call sponge_layer   ( nspl, nspu, csp )
        call poisson
c
        if ( mod(istep,lout) .eq. 0 ) call div_check ( istep )
c
        if ( mod(istep,jout) .eq. 0 ) then
c  ***  Upload of ust for the restart mode  ***
          fub(1,nz+1,1) = ust
c
          call data_output ( nf3d, nf2d )
        end if
c
        aa   = 1.5
        new  = 1
c
      end do
c
      stop
      end
c
      subroutine  div_check ( istep )
c
      include  "Declare.h"
c
      divmax = 0.0
c
      do j = 2,ny-1
      do k = 1,nz
      do i = 2,nx-1
        div = ( u (i+1, k , j )-u (i,k,j) ) / dx
     :       +( v ( i , k ,j+1)-v (i,k,j) ) / dy
     :       +( w ( i ,k+1, j )-w (i,k,j) ) / dz
c
        if ( divmax .lt. abs(div) ) then
          divmax = abs(div)
          imax   = i
          jmax   = j
          kmax   = k
        end if
      end do
      end do
      end do
c
      write(6,*) 'istep,i,j,k,divmax=',istep,imax,jmax,kmax,divmax
c
      return
      end
