c **********************************************************************
c *   Poisson equation for pressure (solved with SMAC method)          *
c *                                                                    *
c *   # The subroutine of FFT is not parallelized.                     *
c **********************************************************************
c
      subroutine  poisson
c
      include  "Declare.h"
c
      dimension  trix(mx), triz(mz), fcos(mz), ifax(20), ifaz(20)
      dimension  wno(mx,mz)
!hpf$ distribute (*,block) :: wno
c
      data  new / 0 /
c
      save  new, trix, triz, fcos, ifax, ifaz, wno
c
      interface
        subroutine tri_diagonal ( uu, wno, dy )
          include  "Implicit.h"
          include  "Parameter.h"
c
          dimension  uu(mx,mz,my), wno(mx,mz)
          intent (in)     dy
          intent (inout)  uu, wno
!hpf$     distribute (*,block,*) :: uu
!hpf$     distribute (*,block)   :: wno
        end subroutine
      end interface
c
      if ( new .ne. 0 ) goto 100
c
        new = 1
c
        call rfftim ( nx-2,       trix, ifax )
        call cosqim ( nz  , fcos, triz, ifaz )
c
        do k = 1,nz
        do i = 2,nx-1
          ix = ( i-1 ) / 2
          zz = ( k-1 ) / 2.0
          xx = sin( 3.14159265358979d0*dble(ix)/dble(nx-2) ) / dx
          zz = sin( 3.14159265358979d0*     zz /dble(nz)   ) / dz
c
c  ***  wno is the square of the modified wavenumber (m^[-2])  ***
c  ***  according to the staggered grid arrangement.           ***
          wno(i,k) = 4.0*( xx**2 +zz**2 )
        end do
        end do
c
  100 continue
c
c  ***  Right-hand side term of the Poisson equation  ***
      do j = 2,ny-1
      do k = 1,nz
      do i = 1,mx
        uu(i,k,j) = -( u (i+1, k , j )-u (i,k,j) ) / dx
     :              -( v ( i , k ,j+1)-v (i,k,j) ) / dy
     :              -( w ( i ,k+1, j )-w (i,k,j) ) / dz
      end do
      end do
      end do
c
c  ***  Forward Fourier transform  ***
      call fft_fb ( trix, triz, fcos, ifax, ifaz,  1 )
c
      call tri_diagonal ( uu, wno, dy )
c
c  ***  Backward Fourier transform  ***
      call fft_fb ( trix, triz, fcos, ifax, ifaz, -1 )
c
      call bc_lat_scalar ( uu )
cx    call bc_upp_scalar ( uu, 0.0 )
c
      pav = 0.0
c
      do j = 2,ny-1
      do i = 2,nx-1
        pav = pav +uu(i,nz,j)
      end do
      end do
c
      pav = pav/dble((nx-2)*(ny-2))
c
c  ***  Correction of the pressure and wind vector  ***
      do j = 1,ny
      do k = 1,nz
      do i = 1,mx
        pd(i,k,j) = pd(i,k,j) +( uu(i,k,j)-pav ) / dt
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 1,nz
        m  = k +1
      do i = 2,nx-1
        u (i,k,j) = u (i,k,j) -( uu(i,k,j)-uu(i-1, k , j ) ) / dx
        v (i,k,j) = v (i,k,j) -( uu(i,k,j)-uu( i , k ,j-1) ) / dy
        w (i,m,j) = w (i,m,j) -( uu(i,m,j)-uu( i ,m-1, j ) ) / dz
      end do
      end do
      end do
c
      call bc_lat_moment
      call bc_upp_moment
c
      return
      end
c
      subroutine  fft_fb ( trix, triz, fcos, ifax, ifaz, ifb )
c
      include  "Declare.h"
c
      dimension  trix(mx), triz(mz), fcos(mz), ifax(20), ifaz(20)
c
      interface
        pure subroutine rfftfm
     :         ( n, inc, jump, lot, r, wa, ifac, wsave )
          include  "Implicit.h"
          dimension  r(*), wsave(*), wa(n), ifac(*)
          intent (in)     n, inc, jump, lot
          intent (inout)  r, wsave, wa, ifac
        end subroutine
c
        pure subroutine rfftbm
     :         ( n, inc, jump, lot, r, wa, ifac, wsave )
          include  "Implicit.h"
          dimension  r(*), wsave(*), wa(n), ifac(*)
          intent (in)     n, inc, jump, lot
          intent (inout)  r, wsave, wa, ifac
        end subroutine
c
        pure subroutine cosqfm
     :         ( n, inc, jump, lot, x, fcos, wa, ifac, wsave )
          include  "Implicit.h"
          dimension  x(*), wsave(*), fcos(n), wa(n), ifac(*)
          intent (in)     n, inc, jump, lot
          intent (inout)  x, wsave, fcos, wa, ifac
        end subroutine
c
        pure subroutine cosqbm
     :         ( n, inc, jump, lot, x, fcos, wa, ifac, wsave )
          include  "Implicit.h"
          dimension  x(*), wsave(*), fcos(n), wa(n), ifac(*)
          intent (in)     n, inc, jump, lot
          intent (inout)  x, wsave, fcos, wa, ifac
        end subroutine
      end interface
c
      nx2 = nx -2
c
      if ( ifb .gt. 0 ) then
c
!hpf$ independent, new(trix, triz, fcos, ifax, ifaz)
        do j = 2,ny-1
          call rfftfm ( nx2, 1,mx,nz ,  uu(2,1,j),
     :                       trix,ifax, vv(2,1,j) )
          call cosqbm ( nz ,mx, 1,nx2,  uu(2,1,j),
     :                  fcos,triz,ifaz, vv(2,1,j) )
        end do
c
      else
c
!hpf$ independent, new(trix, triz, fcos, ifax, ifaz)
        do j = 2,ny-1
          call rfftbm ( nx2, 1,mx,nz ,  uu(2,1,j),
     :                       trix,ifax, vv(2,1,j) )
          call cosqfm ( nz ,mx, 1,nx2,  uu(2,1,j),
     :                  fcos,triz,ifaz, vv(2,1,j) )
        end do
c
      end if
c
      return
      end
c
      subroutine  tri_diagonal ( uu, wno, dy )
c
      include  "Implicit.h"
      include  "Parameter.h"
c
      dimension  cq(mx,my), cs(mx,my), cu(mx,my)
c
      dimension  uu(mx,mz,my), wno(mx,mz)
      intent (in)     dy
      intent (inout)  uu, wno
!hpf$ distribute (*,block,*) :: uu
!hpf$ distribute (*,block)   :: wno
c
      do i = 2,nx-1
        cq(i,   1) = 0.0
        cs(i,   1) = 1.0
        cs(i,ny-1) = 1.0
        cu(i,   1) = 0.0
        cu(i,ny-1) = 0.0
      end do
c
      a  = 1.0 / dy**2
      c  = a
c
!hpf$ independent
      do k = 1,nz
        do j = 2,ny-2
        do i = 2,nx-1
          b  = a +c +wno(i,k)
          d  = uu(i,k,j)
c
          den     =   b -c*cq(i,j-1)
          cq(i,j) =   a                / den
          cu(i,j) = ( d +c*cu(i,j-1) ) / den
          cs(i,j) =      c*cs(i,j-1)   / den
        end do
        end do
c
        do j = ny-2,2,-1
        do i = 2,nx-1
          cs(i,j) = cq(i,j)*cs(i,j+1) +cs(i,j)
          cu(i,j) = cq(i,j)*cu(i,j+1) +cu(i,j)
        end do
        end do
c
        do i = 2,nx-1
          b  = a +c +wno(i,k)
          d  = uu(i,k,ny-1)
c
          uu(i,k,ny-1) = ( d +a*cu(i,2) +c*cu(i,ny-2) )
     :                  /( b -a*cs(i,2) -c*cs(i,ny-2) )
        end do
c
        do j = 2,ny-2
        do i = 2,nx-1
          uu(i,k,j) = cs(i,j)*uu(i,k,ny-1) +cu(i,j)
        end do
        end do
      end do
c
      return
      end
