c **********************************************************************
c *   Time integration for momentum                                    *
c *                                                                    *
c *   Input variables:                                                 *
c *     a             : Coefficient for time integration               *
c *                   : = 1.0,     Euler forward scheme                *
c *                   : = 1.5,     Adams-Bashforth scheme              *
c *     pmx, pmy      : Mean forcing                          (m/s^2)  *
c **********************************************************************
c
      subroutine  predict_moment ( a, pmx, pmy )
c
      include  "Declare.h"
c
      aa  = a
      bb  = 1.0 -aa
c
      if ( fs .eq. 0.0 ) then
        pax = pmx
        pay = pmy
      else
        pax = 0.0
        pay = 0.0
      end if
c
      do j = 1,ny
      do k = 1,nz+1
      do i = 1,mx
        s (i,k,j) = u (i,k,j)
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 1,nz
      do i = 2,nx-1
        fu = ( uu( i , k , j )-uu(i-1, k , j ) ) / dx
     :      +( uv( i , k ,j+1)-uv( i , k , j ) ) / dy
     :      +( wu( i ,k+1, j )-wu( i , k , j ) ) / dz
     :      +fs*( 0.25*( v ( i , k ,j+1)+v (i-1, k ,j+1)
     :                  +v ( i , k , j )+v (i-1, k , j ) )-vg(i,j) )
     :      -fc*  0.25*( w ( i ,k+1, j )+w (i-1,k+1, j )
     :                  +w ( i , k , j )+w (i-1, k , j ) )
c
        u  (i,k,j) = u (i,k,j) +dt*( aa*fu +bb*fub(i,k,j) +pax
     :                       -( pd(i,k,j)-pd(i-1, k , j ) )/dx )
        fub(i,k,j) = fu
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 1,nz
        m  = k +1
      do i = 2,nx-1
        fv = ( uv(i+1, k , j )-uv( i , k , j ) ) / dx
     :      +( vv( i , k , j )-vv( i , k ,j-1) ) / dy
     :      +( vw( i ,k+1, j )-vw( i , k , j ) ) / dz
     :      -fs*( 0.25*( s (i+1, k , j )+s (i+1, k ,j-1)
     :                  +s ( i , k , j )+s ( i , k ,j-1) )-ug(i,j) )
c
        v  (i,k,j) = v (i,k,j) +dt*( aa*fv +bb*fvb(i,k,j) +pay
     :                       -( pd(i,k,j)-pd( i , k ,j-1) )/dy )
        fvb(i,k,j) = fv
c
        fw = ( wu(i+1, m , j )-wu( i , m , j ) ) / dx
     :      +( vw( i , m ,j+1)-vw( i , m , j ) ) / dy
     :      +( ww( i , m , j )-ww( i ,m-1, j ) ) / dz
     :      +( bv( i , m , j )+bv( i ,m-1, j ) ) * 0.5
     :      +fc*  0.25*( s(i+1, m , j )+s(i+1,m-1, j )
     :                  +s( i , m , j )+s( i ,m-1, j ) )
c
        w  (i,m,j) = w (i,m,j) +dt*( aa*fw +bb*fwb(i,m,j)
     :                       -( pd(i,m,j)-pd( i ,m-1, j ) )/dz )
        fwb(i,m,j) = fw
      end do
      end do
      end do
c
c  ***  The following will be carried out in sub.sponge  ***
cx    call bc_lat_moment
cx    call bc_upp_moment
c
      return
      end
