c **********************************************************************
c *   Sub-grid scale (SGS) eddy viscosity model                        *
c *                                                                    *
c *   Input variables:                                                 *
c *     isgs          : = 0,       Standard SGS model                  *
c *                   : otherwise, Sullivan et al. (1994)'s model      *
c *     isfc          : < 0,       for sub.surface_stress_free         *
c *     ust, duz      : obtained from sub.surface_*                    *
c *                                                                    *
c *   # Prandtl number is based on Appendix A in Brown et al. (1994).  *
c **********************************************************************
c
      subroutine  sgs_model ( isgs, isfc, ust, duz )
c
      include  "Declare.h"
c
      dimension  fnu (mz), uav (mz), vav (mz), sdav(mz)
c
      gtr  = g  /tref
      rsum = 1.0/dble((nx-2)*(ny-2))
c
      do k = 1,mz
        gam (k) = 1.0
        fwu (k) = 0.0
        fvw (k) = 0.0
c
        fnu (k) = 0.0
        uav (k) = 0.0
        vav (k) = 0.0
        sdav(k) = 0.0
      end do
c
      do j = 2,ny-1
      do k = 1,nz
      do i = 2,nx-1
        uu(i,k,j) = ( u (i+1, k , j )-u (i,k,j) ) / dx
        vv(i,k,j) = ( v ( i , k ,j+1)-v (i,k,j) ) / dy
        ww(i,k,j) = ( w ( i ,k+1, j )-w (i,k,j) ) / dz
c
        uv(i,k,j) = 0.25*( ( u (i+1, k ,j+1)+u ( i , k ,j+1)
     :                      -u (i+1, k ,j-1)-u ( i , k ,j-1) ) / dy
     :                    +( v (i+1, k ,j+1)+v (i+1, k , j )
     :                      -v (i-1, k ,j+1)-v (i-1, k , j ) ) / dx )
c
        vw(i,k,j) = 0.25*  ( w ( i ,k+1,j+1)+w ( i , k ,j+1)
     :                      -w ( i ,k+1,j-1)-w ( i , k ,j-1) ) / dy
        wu(i,k,j) = 0.25*  ( w (i+1,k+1, j )+w (i+1, k , j )
     :                      -w (i-1,k+1, j )-w (i-1, k , j ) ) / dx
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 2,nz
      do i = 1,mx
        wu(i,k,j) = wu(i,k,j)
     :             +0.25*  ( u (i+1,k+1, j )+u ( i ,k+1, j )
     :                      -u (i+1,k-1, j )-u ( i ,k-1, j ) ) / dz
        vw(i,k,j) = vw(i,k,j)
     :             +0.25*  ( v ( i ,k+1,j+1)+v ( i ,k+1, j )
     :                      -v ( i ,k-1,j+1)-v ( i ,k-1, j ) ) / dz
c
        fh(i,k,j) = ( vt(i,k,j) +0.61*  q (i,k,j) +1.0    )
     :             *0.5 *  ( t ( i ,k+1, j )-t ( i ,k-1, j ) ) / dz
     :             +( vq(i,k,j) +0.61*( t (i,k,j) +tref ) )
     :             *0.5 *  ( q ( i ,k+1, j )-q ( i ,k-1, j ) ) / dz
      end do
      end do
      end do
c
      if ( isfc .ge. 0 ) then
        do j = 2,ny-1
        do i = 2,nx-1
          ugrad = -flm(i,j)*grm(i,j)
          tgrad = -flt(i,j)*grh(i,j)
          qgrad = -flq(i,j)*grh(i,j)
c
          wu(i,1,j) = wu(i,1,j) +0.5*( u(i+1,1, j )+u(i,1,j) )*ugrad
          vw(i,1,j) = vw(i,1,j) +0.5*( v( i ,1,j+1)+v(i,1,j) )*ugrad
c
          fh(i,1,j) = ( vt(i,1,j) +0.61*  q (i,1,j) +1.0    )*tgrad
     :               +( vq(i,1,j) +0.61*( t (i,1,j) +tref ) )*qgrad
        end do
        end do
c
      else
        do j = 2,ny-1
        do i = 2,nx-1
          wu(i,1,j) = wu(i,1,j)
     :               +0.25*  ( u (i+1,2, j )+u ( i ,2, j )
     :                        -u (i+1,1, j )-u ( i ,1, j ) ) / dz
          vw(i,1,j) = vw(i,1,j)
     :               +0.25*  ( v ( i ,2,j+1)+v ( i ,2, j )
     :                        -v ( i ,1,j+1)-v ( i ,1, j ) ) / dz
c
          fh(i,1,j) = ( vt(i,1,j) +0.61*  q (i,1,j) +1.0    )
     :               *0.5 *( ( t (i,2,j)-t (i,1,j) ) / dz + gamt )
     :               +( vq(i,1,j) +0.61*( t (i,1,j) +tref ) )
     :               *0.5 *( ( q (i,2,j)-q (i,1,j) ) / dz + gamq )
        end do
        end do
c
      end if
c
      if ( isgs .eq. 0 ) then
        do j = 2,ny-1
        do k = 1,nz
        do i = 1,mx
          fm(i,k,j) = 2.0*( uu(i,k,j)**2+vv(i,k,j)**2+ww(i,k,j)**2 )
     :                     +uv(i,k,j)**2+vw(i,k,j)**2+wu(i,k,j)**2
        end do
        end do
        end do
c
        do j = 2,ny-1
        do k = 1,nz
        do i = 1,mx
          ri  = 4.0*gtr*fh(i,k,j)/( fm(i,k,j)     +1.0e-20 )
cx        pr  = tpr      +( 1.0-tpr )*min( max( ri, 0.0 ), 1.0 )
          pr  = tpr/( 1.0-( 1.0-tpr )*min( max( ri, 0.0 ), 1.0 ) )
c
          sij = max( fm(i,k,j)      -gtr/pr*fh(i,k,j), 0.0 )
          fme = sqrt( sij )*( csgs*el(i,k,j) )**2
c
c  ***  uu and vv are work variables for sub.statistics.  ***
          uu(i,k,j) = fm(i,k,j)
          vv(i,k,j) = fh(i,k,j)
c
          fm(i,k,j) =   fme
          fh(i,k,j) =   fme/  pr
          tk(i,k,j) = ( fme/( ctk*el(i,k,j) ) )**2
        end do
        end do
        end do
c
      else
        ktop = ktop_search ( 2 )
c
        do j = 2,ny-1
        do k = 1,ktop-1
        do i = 2,nx-1
          fwu(k) = fwu(k) +wu(i,k,j)
          fvw(k) = fvw(k) +vw(i,k,j)
        end do
        end do
        end do
c
        do k = 1,ktop-1
          fwu(k) = fwu(k) * rsum
          fvw(k) = fvw(k) * rsum
        end do
c
        do j = 2,ny-1
        do k = 1,nz
        do i = 2,nx-1
          fm(i,k,j) = 2.0*( uu(i,k,j)**2+vv(i,k,j)**2+ww(i,k,j)**2 )
     :                     +uv(i,k,j)**2+( vw(i,k,j)-fvw(k) )**2
     :                                  +( wu(i,k,j)-fwu(k) )**2
        end do
        end do
        end do
c
        do j = 2,ny-1
        do k = 1,ktop
        do i = 2,nx-1
          uav (k) = uav (k) +u (i,k,j)
          vav (k) = vav (k) +v (i,k,j)
          sdav(k) = sdav(k) +fm(i,k,j)
        end do
        end do
        end do
c
        do k = 1,ktop
          uav (k) = uav (k) * rsum
          vav (k) = vav (k) * rsum
c
          s1av    = sqrt( fwu(k)**2+fvw(k)**2 )
          sdav(k) = sqrt( sdav(k)*rsum ) +1.0e-10
          gam (k) = sdav(k)/( sdav(k)+s1av )
        end do
c
        do k = 2,ktop
          fnu (k) = sqrt( ( uav(k)-uav(k-1) )**2
     :                   +( vav(k)-vav(k-1) )**2 ) / dz
        end do
c
                           gam(ktop) = 1.0
        if ( ktop .gt. 1 ) fnu(  2 ) = duz
        if ( ktop .gt. 2 ) fnu(ktop) = 0.5*fnu(ktop)
c
        do j = 2,ny-1
        do k = 1,nz
          gamk = gam(k)
          s2av = fwu(k)**2 +fvw(k)**2
        do i = 2,nx-1
          ri  = 4.0*gtr*fh(i,k,j)/( fm(i,k,j)+s2av+1.0e-20 )
cx        pr  = tpr      +( 1.0-tpr )*min( max( ri, 0.0 ), 1.0 )
          pr  = tpr/( 1.0-( 1.0-tpr )*min( max( ri, 0.0 ), 1.0 ) )
c
          sij = max( fm(i,k,j)*gamk -gtr/pr*fh(i,k,j), 0.0 )
          fme = sqrt( sij )*( csgs*el(i,k,j) )**2
c
c  ***  uu and vv are work variables for sub.statistics.  ***
          uu(i,k,j) = fm(i,k,j)
          vv(i,k,j) = fh(i,k,j)
c
          fm(i,k,j) =   fme
          fh(i,k,j) =   fme/  pr
          tk(i,k,j) = ( fme/( ctk*el(i,k,j) ) )**2
        end do
        end do
        end do
c
      end if
c
      call bc_lat_scalar ( fm )
      call bc_lat_scalar ( fh )
      call bc_upp_scalar ( fm, 0.0 )
      call bc_upp_scalar ( fh, 0.0 )
c
      if ( isgs .eq. 0 ) return
c
      ufav = 0.0
      vfav = 0.0
      fmav = 0.0
c
      do j = 2,ny-1
      do i = 2,nx-1
        ufav = ufav +0.25*( u (i,2,j)+u ( i , 1 , j ) )
     :                   *( w (i,2,j)+w (i-1, 2 , j ) )
        vfav = vfav +0.25*( v (i,2,j)+v ( i , 1 , j ) )
     :                   *( w (i,2,j)+w ( i , 2 ,j-1) )
        fmav = fmav +0.5 *( fm(i,2,j)+fm( i , 1 , j ) )
      end do
      end do
c
      ufav = ufav * rsum
      vfav = vfav * rsum
      fmav = fmav * rsum * 0.5*( gam(2)+gam(1) )
c
      fms  = max( (ust**2-sqrt(ufav**2+vfav**2))/duz-fmav, 0.0 ) / duz
c
c  ***  Although the expression below is accurate, a computed fms  ***
c  ***  is nealy similar to that from the expression above.        ***
cx    ws   = sqrt( uav(1)**2+vav(1)**2 )
cx    cst  = uav(1)/ws
cx    snt  = vav(1)/ws
cx    f2   = max( ust**4-( ufav*snt-vfav*cst )**2, 0.0 )
cx    fms  = max( (sqrt(f2)+ufav*cst+vfav*snt)/duz-fmav, 0.0 ) / duz
c
      do k = 2,mz
        fmk    = fms*  fnu(k)
        fwu(k) = fmk*( uav(k)-uav(k-1) ) / dz
        fvw(k) = fmk*( vav(k)-vav(k-1) ) / dz
      end do
c
      return
      end
c
c **********************************************************************
c *   Search of the grid number below which Sullivan et al. (1994)'s   *
c *   model is used                                                    *
c **********************************************************************
c
      function  ktop_search ( kst )
c
      include  "Declare.h"
c
      dimension  tfx(mz)
c
      nz2  = nz  *2/3
      tfx0 = 0.0
c
      do k = 1,nz2
        tfx(k) = 0.0
      end do
c
      do j = 2,ny-1
      do k = 1,nz2
      do i = 1,mx
        pp   = 2.0 *ev /( pi0( i ,k+1, j )+pi0(i,k,j) )
        s (i,k,j) = ( t (i,k,j) +tref +pp*ql(i,k,j) )
     :             *( 1.0 +0.61*q (i,k,j) -1.61*ql(i,k,j) )
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 2,nz2
      do i = 2,nx-1
        tfx(k) = tfx(k) +( s (i, k ,j) + s (i,k-1,j) ) * w(i,k,j)
     :                  -( fh(i, k ,j) + fh(i,k-1,j) + 2.0*tnu )
     :            *0.5*( ( vt(i, k ,j) +0.61*  q (i, k ,j) +1.0
     :                    +vt(i,k-1,j) +0.61*  q (i,k-1,j) +1.0    )
     :                  *( t (i, k ,j) - t (i,k-1,j) ) / dz
     :                  +( vq(i, k ,j) +0.61*( t (i, k ,j) +tref )
     :                    +vq(i,k-1,j) +0.61*( t (i,k-1,j) +tref ) )
     :                  *( q (i, k ,j) - q (i,k-1,j) ) / dz )
      end do
      end do
      end do
c
      do j = 2,ny-1
      do i = 2,nx-1
        tfx0 = tfx0 +( vt(i,1,j) +0.61*  q (i,1,j) +1.0    )*flt(i,j)
     :              +( vq(i,1,j) +0.61*( t (i,1,j) +tref ) )*flq(i,j)
      end do
      end do
c
      if ( tfx0 .lt. 0.0 ) then
c
c  ***  Obtains the height at which the heat flux falls to 5%  ***
c  ***  of its surface value (e.g., Andren, 1995).             ***
c
        tfxs = 0.05 * ( 2.0*tfx0 )
c
        do k = 2,nz2
          kk = k
          if ( tfx(k) .gt. tfxs ) exit
        end do
c
      else
c
c  ***  Obtains the height at which the heat flux is minimum.  ***
c
        tfmn = 1.0e10
c
        do k = 2,nz2
          if ( tfx(k) .lt. tfmn ) then
            tfmn = tfx(k)
            kk   = k
          end if
        end do
c
      end if
c
      ktop_search = max( kk/2, kst )
c
      return
      end
