/***********************************************************
 *  A package of 'Calcomp' plotter subroutine (Ver.2)      *
 *                                                         *
 *    For X-window.                                        *
 *                                                         *
 *                        Oct/1995.  coded by M.Nakanishi  *
 ***********************************************************/

#define  CALCOMP  1

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <locale.h>
#include "meta_io.c"
#include "calfunc.h"
#include "calfont.h"

static int  mbak;

/* ---------------------------------------------------------
    X-window open
      x(float) : window width
      y(float) : window height
      m(int)   : do not output META-file when m = 0
 */
void  plots( x, y, m )
    float  *x, *y;
    int    *m;
{
    Window     parent;
    XEvent     event;
    int        screen;
    XGCValues  values;
    unsigned long         valuemask;
    XSetWindowAttributes  attributes;

    float  xx = *x, yy = *y;
    int    mm = (*m)%10;

    setlocale( LC_ALL, "" );

    mndisp = XOpenDisplay( "" );
    if ( !mndisp )
    {
	printf( "Error : Can not open display.\n" ); exit( 1 );
    }

    fpcon = ABS(*m);

    if ( ABS(mm) > 0 && ABS(mm) <= 3 ) { xx = xsize[mm+3]; yy = ysize[mm+3]; }

    mnwx  = xx*mnwfac +0.5;
    mnwy  = yy*mnwfac +0.5;
    if ( mnwx > WMAX || mnwy > WMAX )
    {
	mnwfac = mnwfac*WMAX/MAX( mnwx, mnwy );
	printf( "Window factor was changed to %f.\n", mnwfac );
    }

    if ( fpcon%10 ) io_plots( &xx, &yy, m, &mnwfac );

    mnwx  = xx*mnwfac +1.5;   /* EBhE̓sNZPʂȂ̂ */
    mnwy  = yy*mnwfac +1.5;   /* 1                      */
    mnx0  = 0.0;
    mny0  = mnwy -1;
    mnfac = mnwfac;
    mnlw  = 0;

    screen = DefaultScreen( mndisp );
    parent = RootWindow( mndisp, screen );
    mncmap = DefaultColormap( mndisp, screen );
    mnfrc  = BlackPixel( mndisp, screen );
    mnbkc  = WhitePixel( mndisp, screen );

    mnwin  = XCreateSimpleWindow( mndisp, parent, 20, 20, mnwx, mnwy, 5,
                                  mnfrc, mnbkc );

    attributes.bit_gravity = StaticGravity;
    attributes.event_mask  = ExposureMask | KeyPressMask |
	ButtonMotionMask | ButtonPressMask | ButtonReleaseMask;

    XChangeWindowAttributes( mndisp, mnwin,
	CWBitGravity | CWEventMask, &attributes );
    XStoreName( mndisp, mnwin, mnname );

    values.foreground = mnfrc;
    values.background = mnbkc;
    values.join_style = JSTYL;
    valuemask = GCForeground | GCBackground | GCJoinStyle;

    mngcd  = XCreateGC( mndisp, mnwin, valuemask, &values );
    mngcf  = XCreateGC( mndisp, mnwin, valuemask, &values );

    mnpix  = XCreatePixmap( mndisp, mnwin, mnwx, mnwy,
                            DefaultDepth( mndisp, screen ) );

    if ( fpcon < 10 )
    {
	XMapWindow( mndisp, mnwin );
	XNextEvent( mndisp, &event );
    }
    else
    {
	XUnmapWindow( mndisp, mnwin );

	mnwin = mnpix;
	XSetForeground( mndisp, mngcd, mnbkc );
	XFillRectangle( mndisp, mnwin, mngcd, 0, 0, mnwx, mnwy );
	XSetForeground( mndisp, mngcd, mnfrc );

	XFlush( mndisp );
    }
}

/* ---------------------------------------------------------
    X-window close
 */
void  plote()
{
    if ( fpcon%10  ) io_plote();

    XFreeGC( mndisp, mngcd );
    XFreeGC( mndisp, mngcf );
    XFreePixmap( mndisp, mnpix );
    XCloseDisplay( mndisp );
}

/* ---------------------------------------------------------
    line plotting & pen moving
      x(float) : end position of the pen (x-coordinate)
      y(float) : end position of the pen (y-coordinate)
      m(int)   : pen down when m = 2
 */
void  plot( x, y, m )
    float  *x, *y;
    int    *m;
{
    int    ixe, iye;

    if ( fpcon%10 ) io_plot( x, y, m );

    ixe = XPOS( *x, mnfac, mnx0 );
    iye = YPOS( *y, mnfac, mny0 );

    if ( ABS(*m) == 2 )
	XDrawLine( mndisp, mnwin, mngcd, mnxs, mnys, ixe, iye );

    mnxs = ixe;
    mnys = iye;

    if ( *m < 0 )
    {
	mnx0 = mnx0 +(*x)*mnfac;
	mny0 = mny0 -(*y)*mnfac;
    }
}

/* ---------------------------------------------------------
    change color
      m(int)   : m = red*100 + green*10 + blue
                 0(dark) =< red,green,blue =< 9(light)
                 Background color when m < 0
 */
#define  MBIT  65535

void  color( m )
    int  *m;
{
    XColor  colv;
    int     mm, cr, cg, cb;

    if ( fpcon%10 ) io_color( m );

    mm = ABS(*m); if ( *m < 0 ) mm--;
    if ( mm > 999 ) { mndiv = mm/1000; return; }

    cr =   mm/100;
    cg = ( mm%100 )/10;
    cb =   mm%10;
    colv.red   = MBIT*( cr/9.0 );
    colv.green = MBIT*( cg/9.0 );
    colv.blue  = MBIT*( cb/9.0 );

    if ( !XAllocColor( mndisp, mncmap, &colv ) )
    {
	printf( "Warning : Can not allocate color(%d).\n", mm );
	return;
    }

    if ( *m >= 0 )
    {
	mnfrc = colv.pixel;
	XSetForeground( mndisp, mngcd, mnfrc );
	XSetForeground( mndisp, mngcf, mnfrc );
    }
    else
    {
	mnbkc = colv.pixel;
	XSetBackground( mndisp, mngcd, mnbkc );
	XSetBackground( mndisp, mngcf, mnbkc );

	if ( fpcon < 10 )
	{
	    XSetWindowBackground( mndisp, mnwin, mnbkc );
	    XClearWindow( mndisp, mnwin );
	}
	else
	{
	    XSetForeground( mndisp, mngcd, mnbkc );
	    XFillRectangle( mndisp, mnwin, mngcd, 0, 0, mnwx, mnwy );
	    XSetForeground( mndisp, mngcd, mnfrc );
	}
    }
}

void  color2( cr, cg, cb )
    int  *cr, *cg, *cb;
{
    XColor  colv;
    unsigned long  bit = 256;

    if ( fpcon%10 ) io_color2( cr, cg, cb );

    colv.red   = MIN( bit*(*cr), MBIT );
    colv.green = MIN( bit*(*cg), MBIT );
    colv.blue  = MIN( bit*(*cb), MBIT );

    if ( !XAllocColor( mndisp, mncmap, &colv ) )
    {
	printf( "Warning : Can not allocate color2(%d,%d,%d).\n",
	         cr,cg,cb ); return;
    }

    mnfrc = colv.pixel;
    XSetForeground( mndisp, mngcd, mnfrc );
    XSetForeground( mndisp, mngcf, mnfrc );
}

/* ---------------------------------------------------------
    change line_style & line_width
      m(int)   : m = style*10 + width
                 style = 0(solid line)
                         1(dashed line )
                         2(dotted line )
                         3(one dotted dashed line )
                         4(two dotted dashed line )
                 width = 0(by X-server), 1 to 7 pixels
 */
void  newpen( m )
    int  *m;
{
    int          ls = LineSolid, mm;
    static char  dash1[3][4] = { 4,4,4,4, 1,3,1,3, 8,2,2,2 },
                 dash2[] = { 8,2,1,2,1,2 };

    if ( fpcon%10 ) io_newpen( m );

    mm = ABS(*m);

/*
    mnlw = MIN( mm%10, 7 );
 */
    mnlw = mm%10; if ( mnlw > 4 ) mnlw = 0;
    mm   = mm/10;

    if ( mm )
    {
	ls = LineOnOffDash;
	if ( mm < 4 ) XSetDashes( mndisp, mngcd, 0, dash1[mm-1], 4 );
	else          XSetDashes( mndisp, mngcd, 0, dash2,       6 );
    }

    XSetLineAttributes( mndisp, mngcd, mnlw, ls, CapButt, JSTYL );
}

/* ---------------------------------------------------------
    clear graphics
      m(int)   : waiting when m > 0
 */
void  cls( m )
    int  *m;
{
    if ( fpcon%10 ) io_cls( m );

    if ( *m > 0 && fpcon < 10 )
    {
	XEvent  event;
	Pixmap  pix2;
	int     screen;

	screen = DefaultScreen( mndisp );
	pix2   = XCreatePixmap( mndisp, mnwin, mnwx, mnwy,
	                        DefaultDepth( mndisp, screen ) );
	XCopyArea( mndisp, mnwin, pix2, mngcd, 0, 0, mnwx, mnwy, 0, 0 );

	do {
	    XNextEvent( mndisp, &event );

	    if ( event.type == Expose )
	      XCopyArea( mndisp, pix2, mnwin, mngcd, 0, 0, mnwx, mnwy, 0, 0 );
	}
	while ( event.type != KeyPress && event.type != ButtonPress );

	XFreePixmap( mndisp, pix2 );
    }

    if ( *m >= 0 )
    {
	if ( fpcon < 10 )
	    XClearWindow( mndisp, mnwin );
	else
	{
	    XSetForeground( mndisp, mngcd, mnbkc );
	    XFillRectangle( mndisp, mnwin, mngcd, 0, 0, mnwx, mnwy );
	    XSetForeground( mndisp, mngcd, mnfrc );
	}
    }

    XFlush( mndisp );
}

/* ---------------------------------------------------------
    get graphic image
      xs(float) : lower left  x-coordinate
      ys(float) : lower left  y-coordinate
      xe(float) : upper right x-coordinate
      ye(float) : upper right y-coordinate
 */
void  get( xs, ys, xe, ye )
    float  *xs, *ys, *xe, *ye;
{
    int    ixs, iys, ixe, iye;
    int    wx = mnwx-1, wy = mnwy-1;

    if ( fpcon%10 ) io_get( xs, ys, xe, ye );
/*
    ixs  =     (int)((*xs)*mnwfac+0.5); if ( ixs <  0 ) ixs = 0;
    iys  = wy -(int)((*ye)*mnwfac+0.5); if ( iys <  0 ) iys = 0;
    ixe  =     (int)((*xe)*mnwfac+0.5); if ( ixe > wx ) ixe = wx;
    iye  = wy -(int)((*ys)*mnwfac+0.5); if ( iye > wy ) iye = wy;
 */
    ixs  = XPOS( *xs, mnfac, mnx0 ); if ( ixs <  0 ) ixs = 0;
    iys  = YPOS( *ye, mnfac, mny0 ); if ( iys <  0 ) iys = 0;
    ixe  = XPOS( *xe, mnfac, mnx0 ); if ( ixe > wx ) ixe = wx;
    iye  = YPOS( *ys, mnfac, mny0 ); if ( iye > wy ) iye = wy;

    mnpw = ixe -ixs +1;   /* EBhEƓl̗Rɂ */
    mnph = iye -iys +1;   /* 1                    */

    XCopyArea( mndisp, mnwin, mnpix, mngcd, ixs, iys, mnpw, mnph, 0, 0 );
}

/* ---------------------------------------------------------
    put graphic image
      x(float) : lower left  x-coordinate
      y(float) : lower left  y-coordinate
 */
void  put( x, y )
    float  *x, *y;
{
    int    ix, iy;
    int    wy = mnwy-1;

    if ( fpcon%10 ) io_put( x, y );
/*
    ix =     (int)((*x)*mnwfac+0.5); if ( ix <  0 ) ix = 0;
    iy = wy -(int)((*y)*mnwfac+0.5); if ( iy > wy ) iy = wy;
 */
    ix = XPOS( *x, mnfac, mnx0 ); if ( ix <  0 ) ix = 0;
    iy = YPOS( *y, mnfac, mny0 ); if ( iy > wy ) iy = wy;
    iy = iy -mnph +1;

    XCopyArea( mndisp, mnpix, mnwin, mngcd, 0, 0, mnpw, mnph, ix, iy );
    XFlush( mndisp );
}

/* ---------------------------------------------------------
    draw string
      x(float)  : plot start (x-coordinate)
      y(float)  : plot start (y-coordinate)
      h(float)  : string height
      ch(char)  : string
      th(float) : string angle (degree)
      n(int)    : string length
 */
static int  mnbite = 1;

void  symbol( x, y, h, ch, th, n )
    float  *x, *y, *h, *th;
    char   ch[];
    int    *n;
{
#ifdef  R4
    XFontStruct  *fst;
    XCharStruct  cst;
    int    direction, ascent;
#else
    XFontSet     font;
    XRectangle   ink, log;
    char   **miss, *def;
    int    n_miss;
#endif
    int    ix, iy, jx, jy, width, height, descent;
    int    nn, dot;
    register int  i, j;

    if ( fpcon%10 ) io_symbol( x, y, h, ch, th, n );

    nn = MAX( (*n)%100, 1 );

    if ( *x > 9998.9*mnwfac/mnfac ) ix = mnxs;
    else                            ix = XPOS( *x, mnfac, mnx0 );
    if ( *y > 9998.9*mnwfac/mnfac ) iy = mnys;
    else                            iy = YPOS( *y, mnfac, mny0 );

    dot = ABS(*h)*mnfac +0.5;

    for ( i = 0 ; i < FNUM-1 ; i++ )
	if ( font_dot[i] >= dot ) break;

#ifdef  R4
    fst = XLoadQueryFont( mndisp, font_name[i] );
    XSetFont( mndisp, mngcd, fst->fid );
    XTextExtents( fst, ch, nn, &direction, &ascent, &descent, &cst );

    width   = cst.width;
    height  = ascent +descent;

    if ( *n > 0 )
    {
	jx = ix;
	jy = iy -descent;
    }
    else   /* Centered symbol */
    {
	jx = ix -width/2;
	jy = iy +( cst.ascent-cst.descent )/2;
	width = 0;
    }
#else
    font = XCreateFontSet( mndisp, font_name[i], &miss, &n_miss, &def );
    XmbTextExtents( font, ch, nn, &ink, &log );

    width   = log.width;
    height  = log.height;
    descent = height +log.y;

    if ( *n > 0 )
    {
	jx = ix;
	jy = iy -descent;
    }
    else   /* Centered symbol */
    {
	jx = ix -width/2;
	jy = iy -ink.y -ink.height/2;
	width = 0;
    }
#endif

    if ( *th == 0.0 || *n < 1 )
    {
#ifdef  R4
	XDrawString( mndisp, mnwin, mngcd, jx, jy, ch, nn );
#else
	XmbDrawString( mndisp, mnwin, font, mngcd, jx, jy, ch, nn );
#endif
	mnxs = ix +width;
	mnys = iy;
    }
    else
    {
	double  ay, ky = jy;
	int     ax = 1;

	if      ( *th <    0.0 ) { jy += height; ky = jy; }

	if      ( *th ==  90.0 ) { ax = 0; ay =  1.0; }
	else if ( *th == -90.0 ) { ax = 0; ay = -1.0; }
	else                       ay = tan( (double)(*th)*DTR );

	for ( j = 0 ; j < nn ; j += mnbite )
	{
#ifdef  R4
	    width = XTextWidth( fst, &ch[j], 1 );
	    XDrawString( mndisp, mnwin, mngcd, jx, jy, &ch[j], 1 );
#else
	    XmbTextExtents( font, &ch[j], mnbite, &ink, &log );
	    width = log.width;
	    XmbDrawString( mndisp, mnwin, font, mngcd, jx, jy, &ch[j], mnbite );
#endif
	    if ( ax != 0 ) height = width;

	    jx = jx +ax*width;
	    ky = ky -ay*height; jy = ky +(ky > 0.0 ? 0.5 : -0.5);
	}

	mnxs = jx;
	mnys = jy +descent;
    }
}

/* ---------------------------------------------------------
    draw Japanese
 */
void  nkanji( x, y, h, ch, th, n )
    float  *x, *y, *h, *th;
    char   ch[];
    int    *n;
{
#ifdef  R4
    XFontStruct  *fst;
    XCharStruct  cst;
    int    direction, ascent, descent;
    int    ix, iy, jx, jy, width, height;
    int    nn, no, dot;
    register int  i, j;

    unsigned char  jis[CMAX*2+1];
    float  dr;

    if ( fpcon%10 ) io_nkanji( x, y, h, ch, th, n );

    nn = ABS(*n)%100; no = 2*MIN( nn, CMAX );
    dr = *th; if ( *n < 0 ) dr -= 90.0;

    if ( *x > 9998.9*mnwfac/mnfac ) ix = mnxs;
    else                            ix = XPOS( *x, mnfac, mnx0 );
    if ( *y > 9998.9*mnwfac/mnfac ) iy = mnys;
    else                            iy = YPOS( *y, mnfac, mny0 );

    strncpy( (char *)jis, ch, no ); jis[no] = '\0';
#if  SJIS
    sj2jis ( jis );
#else
    euc2jis( jis );
#endif

    dot = ABS(*h)*mnfac +0.5;

    for ( i = FNUM ; i < FNUM+JNUM-1 ; i++ )
	if ( font_dot[i] >= dot ) break;

    fst = XLoadQueryFont( mndisp, font_name[i] );
    XSetFont( mndisp, mngcd, fst->fid );
    XTextExtents16( fst, jis, nn, &direction, &ascent, &descent, &cst );

    width  = cst.width;
    height = ascent +descent;

    jx = ix;
    jy = iy -descent;
 
    if ( dr == 0.0 )
    {
	XDrawString16( mndisp, mnwin, mngcd, jx, jy, jis, nn );

	mnxs = ix +width;
	mnys = iy;
    }
    else
    {
	double  ay, ky = jy;
	int     ax = 1;

	if      ( dr <    0.0 ) { jy += height; ky = jy; }

	if      ( dr ==  90.0 ) { ax = 0; ay =  1.0; }
	else if ( dr == -90.0 ) { ax = 0; ay = -1.0; }
	else                      ay = tan( (double)dr*DTR );

        for ( j = 0 ; j < nn ; j++ )
        {
	    width = XTextWidth16( fst, &jis[j*2], 1 );
	    XDrawString16( mndisp, mnwin, mngcd, jx, jy, &jis[j*2], 1 );

	    if ( ax != 0 ) height = width;

	    jx = jx +ax*width;
	    ky = ky -ay*height; jy = ky +(ky > 0.0 ? 0.5 : -0.5);
        }

	mnxs = jx;
	mnys = jy +descent;
    }
#else
    int    no;
    float  dr;

    no = ABS(*n); no = no +no%100;
    dr = *th; if ( *n < 0 ) dr -= 90.0;

    mnbite = 2;
    symbol( x, y, h, ch, &dr, &no );
    mnbite = 1;
#endif
}

static unsigned char stiple[][8] = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x88,0x00,0x22,0x00,0x88,0x00,0x22,0x00,
	0xaa,0x00,0xaa,0x00,0xaa,0x00,0xaa,0x00,
	0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,
	0xaa,0xff,0xaa,0xff,0xaa,0xff,0xaa,0xff,
	0xee,0xff,0xbb,0xff,0xee,0xff,0xbb,0xff
	};

void  mnstyle( m )
    int  m;
{
    int  mm = ABS(m), nn;

    if ( mm == mbak ) return;
    mbak =   mm;
    nn   = ( mm-1 )%10;

    if ( nn >= 6 )
	XSetFillStyle( mndisp, mngcf, FillSolid );
    else
    {
	Pixmap  checker;
	int     style = FillStippled;

	if ( mm < 10 ) style = FillOpaqueStippled;

	checker = XCreateBitmapFromData( mndisp, mnwin, stiple[nn], 8, 8 );
	XSetStipple( mndisp, mngcf, checker );
	XSetFillStyle( mndisp, mngcf, style );
	XFreePixmap( mndisp, checker );
    }
}

/* ---------------------------------------------------------
    draw polygon
      x(float) : polygon points (x-coordinate)
      y(float) : polygon points (y-coordinate)
      n(int)   : the number of points
      m(int)   : the pattern of fill area when m != 0
 */
#define  SHAPE  Nonconvex   /* or Complex */

void  polygn( x, y, n, m )
    float  x[], y[];
    int    *n, *m;
{
    XPoint  points[PMAX+1];
    int     nn = *n;
    register int  i;

    if ( nn > PMAX )
    {
	printf( "Warning : n = %d > PMAX in polygn\n", nn ); return;
    }

    if ( fpcon%10 ) io_polygn( x, y, n, m );

    for ( i = 0 ; i < nn ; i++ )
    {
	points[i].x = XPOS( x[i], mnfac, mnx0 );
	points[i].y = YPOS( y[i], mnfac, mny0 );
    }

    if ( *m )
    {
	mnstyle( *m );

	XFillPolygon( mndisp, mnwin, mngcf, points, nn,
	              SHAPE, CoordModeOrigin );

	points[nn].x = points[0].x;
	points[nn].y = points[0].y; nn++;
    }
    if ( *m >= 0 )
	XDrawLines( mndisp, mnwin, mngcd, points, nn, CoordModeOrigin );
}

/* ---------------------------------------------------------
    draw circle
      x(float)  : circle centered points (x-coordinate)
      y(float)  : circle centered points (y-coordinate)
      rx(float) : radius for x-coordinate
      ry(float) : radius for y-coordinate
      as(float) : start angle (degree)
      ae(float) : end   angle (degree)
      m(int)    : the pattern of fill area when m != 0
 */
void  circle( x, y, rx, ry, as, ae, m )
    float  *x, *y, *rx, *ry, *as, *ae;
    int    *m;
{
    int    ix, iy, iw, ih, ias, iae;

    if ( fpcon%10 ) io_circle( x, y, rx, ry, as, ae, m );

    iw  = (*rx)*mnfac +0.5;
    ih  = (*ry)*mnfac +0.5;
    ix  = XPOS( *x, mnfac, mnx0 ) -iw;
    iy  = YPOS( *y, mnfac, mny0 ) -ih;
    iw  = 2*iw;
    ih  = 2*ih;
    ias = (*as)*64.0      +0.5;
    iae = (*ae)*64.0 -ias +0.5;

    if ( *m )
    {
	mnstyle( *m );
	XFillArc( mndisp, mnwin, mngcf, ix, iy, iw, ih, ias, iae );
    }
    if ( *m >= 0 )
	XDrawArc( mndisp, mnwin, mngcd, ix, iy, iw, ih, ias, iae );
}

void  circls( x, y, n, rx, ry, as, ae, m )
    float  x[], y[], *rx, *ry, *as, *ae;
    int    *n, *m;
{
    XArc   arcs[PMAX];
    int    iw, ih, ias, iae;
    register int  i;

    if ( *n > PMAX )
    {
	printf( "Warning : n = %d > PMAX in circls\n", *n ); return;
    }

    if ( fpcon%10 ) io_circls( x, y, n, rx, ry, as, ae, m );

    iw  = (*rx)*mnfac +0.5;
    ih  = (*ry)*mnfac +0.5;
    ias = (*as)*64.0      +0.5;
    iae = (*ae)*64.0 -ias +0.5;

    for ( i = 0 ; i < *n ; i++ )
    {
	arcs[i].x = XPOS( x[i], mnfac, mnx0 ) -iw;
	arcs[i].y = YPOS( y[i], mnfac, mny0 ) -ih;
	arcs[i].width  = 2*iw;
	arcs[i].height = 2*ih;
	arcs[i].angle1 = ias;
	arcs[i].angle2 = iae;
    }

    if ( *m )
    {
	mnstyle( *m );
	XFillArcs( mndisp, mnwin, mngcf, arcs, *n );
    }
    if ( *m >= 0 )
	XDrawArcs( mndisp, mnwin, mngcd, arcs, *n );
}

/* ---------------------------------------------------------
    draw rectangle
      x(float) : rectangle lower left x-coordinate
      y(float) : rectangle lower left y-coordinate
      w(float) : rectangle width
      h(float) : rectangle height
      r(float) : corner radius
      m(int)   : the pattern of fill area when m != 0
 */
void  rect( x, y, w, h, r, m )
    float  *x, *y, *w, *h, *r;
    int    *m;
{
    int    ix, iy, jx, jy;
    unsigned int  iw, ih;

    if ( fpcon%10 ) io_rect( x, y, w, h, r, m );

    ix = XPOS(  *x,       mnfac, mnx0 );
    iy = YPOS( (*y)+(*h), mnfac, mny0 );
    jx = XPOS( (*x)+(*w), mnfac, mnx0 );
    jy = YPOS(  *y,       mnfac, mny0 );
    iw = jx -ix;   /* XFillRectangle ł́C{ */
    ih = jy -iy;   /* iw+1, ih+1 Ƃ悢   */

    if ( *r <= 0.0 )
    {
	if ( *m )
	{
	    mnstyle( *m );
	    XFillRectangle( mndisp, mnwin, mngcf, ix, iy, iw, ih );
	}
	if ( *m >= 0 )
	    XDrawRectangle( mndisp, mnwin, mngcd, ix, iy, iw, ih );
    }
    else
    {
	int  ag = 90*64;
	unsigned int  ir, jr, rmax = MIN(iw,ih)/2;

	ir = (*r)*mnfac +0.5; if ( ir > rmax ) ir = rmax;
	jr = 2*ir;

	if ( *m )
	{
	    mnstyle( *m );

	    XFillRectangle( mndisp, mnwin, mngcf, ix   ,iy+ir, iw, ih-jr );
	    XFillRectangle( mndisp, mnwin, mngcf, ix+ir,iy   , iw-jr, ih );

	    XFillArc( mndisp, mnwin, mngcf, jx-jr,iy   , jr,jr,    0,ag );
	    XFillArc( mndisp, mnwin, mngcf, ix   ,iy   , jr,jr,   ag,ag );
	    XFillArc( mndisp, mnwin, mngcf, ix   ,jy-jr, jr,jr, 2*ag,ag );
	    XFillArc( mndisp, mnwin, mngcf, jx-jr,jy-jr, jr,jr, 3*ag,ag );
	}
	if ( *m >= 0 )
	{
	    XDrawLine( mndisp, mnwin, mngcd, ix   , iy+ir, ix   , jy-ir );
	    XDrawLine( mndisp, mnwin, mngcd, jx   , iy+ir, jx   , jy-ir );
	    XDrawLine( mndisp, mnwin, mngcd, ix+ir, iy   , jx-ir, iy    );
	    XDrawLine( mndisp, mnwin, mngcd, ix+ir, jy   , jx-ir, jy    );

	    XDrawArc( mndisp, mnwin, mngcd, jx-jr,iy   , jr,jr,    0,ag );
	    XDrawArc( mndisp, mnwin, mngcd, ix   ,iy   , jr,jr,   ag,ag );
	    XDrawArc( mndisp, mnwin, mngcd, ix   ,jy-jr, jr,jr, 2*ag,ag );
	    XDrawArc( mndisp, mnwin, mngcd, jx-jr,jy-jr, jr,jr, 3*ag,ag );
	}
    }
}

/* ---------------------------------------------------------
    Child-window open
      x(float) : window upper left x-coordinate
      y(float) : window upper left y-coordinate
      w(float) : window width
      h(float) : window height
 */
void  childs( x, y, w, h )
    float  *x, *y, *w, *h;
{
    unsigned int  wx, wy, xs, ys;

    if ( fpcon%10 ) io_childs( x, y, w, h );
/*
    wx = (*w)*mnwfac +1.5;
    wy = (*h)*mnwfac +1.5;
    xs = (*x)*mnwfac +0.5;
    ys = (*y)*mnwfac +0.5;
 */
    wx = (*w)*mnfac +1.5;
    wy = (*h)*mnfac +1.5;
    xs = XPOS( *x, mnfac, mnx0 );
    ys = YPOS( *y, mnfac, mny0 );
    if ( xs+wx+10 > mnwx ) xs = mnwx-wx-10;
    if ( ys+wy+10 > mnwy ) ys = mnwy-wy-10;

    mnx0bak  = mnx0;
    mny0bak  = mny0;
    mnfacbak = mnfac;
    mnx0     = 0.0;
    mny0     = wy -1;

    mnwinbak = mnwin;
    mnwin    = XCreateSimpleWindow( mndisp, mnwinbak, xs, ys, wx, wy, 5,
                                    mnfrc, mnbkc );

    XMapWindow( mndisp, mnwin );
    XFlush( mndisp );
}

void  childe()
{
    if ( fpcon%10 ) io_childe();

    XDestroyWindow( mndisp, mnwin );

    mnx0  = mnx0bak;
    mny0  = mny0bak;
    mnfac = mnfacbak;
    mnwin = mnwinbak;
}

void  newfnt( m )
    int  *m;
{
    if ( fpcon%10 ) io_newfnt( m );
}

void  rectpt( alt, mx, nx, ny, x, y, h, c, n, zn )
    float  *alt, x[], y[], h[], *zn;
    int    *mx, *nx, *ny, c[], *n;
{
    XImage  *image;
    XColor  colv;
    int     screen;
    int     image_size, *ix, *iy, jx, jy, sg = (*zn > 0.0 ? -1 : 1);
    float   grd, tzn;
    unsigned long   *pix, *kp;
    register int    i, j, k, l;
    register float  *alt_p;

    if ( fpcon%10 ) io_rectpt( alt, mx, nx, ny, x, y, h, c, n, zn );

    screen = DefaultScreen( mndisp );

    image  = XCreateImage( mndisp, DefaultVisual(mndisp,screen),
	DefaultDepth(mndisp,screen), ZPixmap, 0, 0, mnwx, mnwy, 32, 0 );
    image_size  = mnwy * image->bytes_per_line;

    pix = malloc( (*n)*2  * sizeof(long) );
    ix  = malloc( (*nx+1) * sizeof(int) );
    iy  = malloc( (*ny+1) * sizeof(int) );
    image->data = malloc( image_size );

    for ( i = 0 ; i < *n ; i++ ) {
	colv.red   =   c[i]/10000L        *MBIT/99.0;
	colv.green = ( c[i]%10000L )/100L *MBIT/99.0;
	colv.blue  =   c[i]%100L          *MBIT/99.0;

	XAllocColor( mndisp, mncmap, &colv );
	pix[i*2]   = colv.pixel;

	colv.red   *= 0.66;
	colv.green *= 0.66;
	colv.blue  *= 0.66;

	XAllocColor( mndisp, mncmap, &colv );
	pix[i*2+1] = colv.pixel;
    }

    jx = XPOS( x[0],     mnfac, mnx0 ); ix[0]   = MAX(MIN(jx,mnwx),0);
    jx = XPOS( x[*nx-1], mnfac, mnx0 ); ix[*nx] = MAX(MIN(jx,mnwx),0);
    jy = YPOS( y[0],     mnfac, mny0 ); iy[0]   = MAX(MIN(jy,mnwy),0);
    jy = YPOS( y[*ny-1], mnfac, mny0 ); iy[*ny] = MAX(MIN(jy,mnwy),0);

    for ( i = 1 ; i < *nx ; i++ ) {
	jx    = XPOS( 0.5*(x[i-1]+x[i]), mnfac, mnx0 );
	ix[i] = MAX( MIN( jx,mnwx ),0 );
    }
    for ( j = 1 ; j < *ny ; j++ ) {
	jy    = YPOS( 0.5*(y[j-1]+y[j]), mnfac, mny0 );
	iy[j] = MAX( MIN( jy,mnwy ),0 );
    }

    tzn = sg*tan( (double)(*zn+90.0)*DTR );

    for ( j = 0 ; j < *ny ; j++ ) {
	alt_p = alt + j*(*mx);

	for ( i = 0 ; i < *nx ; i++,alt_p++ ) {
	    for ( k = 0 ; k < *n ; k++ )
		if ( h[k] > *alt_p ) break;
	    if ( k == 0 ) continue;

	    if      ( i ==     0 )
		grd = ( *(alt_p+1) - *alt_p     )/( x[i+1]-x[i]   );
	    else if ( i == *nx-1 )
		grd = ( *alt_p     - *(alt_p-1) )/( x[i]  -x[i-1] );
	    else
		grd = ( *(alt_p+1) - *(alt_p-1) )/( x[i+1]-x[i-1] );

	    kp = &pix[2*(k-1)]; if ( sg*grd > tzn ) kp++;

	    for ( l = iy[j+1] ; l < iy[j] ; l++ )
		for ( k = ix[i] ; k < ix[i+1] ; k++ )
		    XPutPixel( image, k, l, *kp );
	}
    }

    XPutImage( mndisp, mnwin, mngcd, image, ix[0], iy[*ny], ix[0], iy[*ny],
	       ix[*nx]-ix[0], iy[0]-iy[*ny] );

    free ( pix );
    free ( ix );
    free ( iy );
    free ( image->data );
    XFree( image );
}

/* ---------------------------------------------------------
    save or restore the current color
 */
void  colsr( m )
    int  *m;
{
    static unsigned long  mnfrc_c;

    if ( fpcon%10 ) io_colsr( m );

    if ( *m ) {
	mnfrc_c = mnfrc;
    }
    else {
	mnfrc = mnfrc_c;
	XSetForeground( mndisp, mngcd, mnfrc );
	XSetForeground( mndisp, mngcf, mnfrc );
    }
}

void  rclip( m, x, y, w, h )
    float  *x, *y, *w, *h;
    int    *m;
{
    if ( fpcon%10 ) io_rclip( m, x, y, w, h );

    if ( !(*m) ) {
	XSetClipMask( mndisp, mngcd, None );
	XSetClipMask( mndisp, mngcf, None );
    }
    else {
	int  ix, iy, jx, jy;
	XRectangle  rect;

	ix = XPOS(  *x,       mnfac, mnx0 );
	iy = YPOS( (*y)+(*h), mnfac, mny0 );
	jx = XPOS( (*x)+(*w), mnfac, mnx0 );
	jy = YPOS(  *y,       mnfac, mny0 );

	rect.x = ix;
	rect.y = iy;
	rect.width  = jx -ix;
	rect.height = jy -iy;

	XSetClipRectangles( mndisp, mngcd, 0, 0, &rect, 1, Unsorted );
	XCopyGC( mndisp, mngcd, GCClipMask, mngcf );
    }
}
