c **********************************************************************
c *   Vertical profiles of inflection points                           *
c *                                                                    *
c *                                   Nov/2006  M. Nakanishi (N.D.A)   *
c **********************************************************************
c
      include  "sub/graph.f"
      include  "../src/Parameter.h"
c
      parameter ( mf=2, mt=9 )
      character (len=4)  :: f_name(mf)
      character (len=16) :: e_lab
      character (len=10) :: z_lab
      character (len=5)  :: r_name
      double precision  dat(mx,mz,my), ave
      dimension  vpro(mz,mf), tpro(mz), th(mt)
      dimension  xx(mz), yy(mz)
c
      data  f_name/ 'u___', 'v___'/
      data  e_lab / 'Lateral velocity'/
      data  z_lab / 'Height (m)'/
      data  th    /-90.0,-45.0,-30.0,-15.0,  0.0,
     :              15.0, 30.0, 45.0, 90.0/
      data  xsta  /-40.0/
      data  xend  / 40.0/
c
      namelist  /les2/  isgs,  isfc,  irad,  icon,  isca,
     :                  jmin,  rlat,    dt,    dx,    dy,    dz,
     :                         ugsf,  vgsf,         aglw,  agmd,
     :                  gulw,  gvlw,  gumd,  gvmd,
     :                  tref,   tsf,   qsf,   psf,  hglw,  hgmd,
     :                  gtlw,  gqlw,  gtmd,  gqmd,  gamt,  gamq,
     :                   z0m,   z0h,  tfmx,  qfmx, phour, dhour,
     :                 tdeep,  swet,  crso,  fkso,
     :                  elat,  albd,  clov,  ulov,
     :                 nflay, frate,
     :                  nspl,  nspu,   csp, nstot
c
      write(6,*) 'Run + Restart number (xx.xx) =?'
      read (5,'(a)')  r_name
      write(6,*) 'Record number =?'
      read (5,*)  idmy, idmy, itplt
c
      open (10,file='para'//r_name,status='old',iostat=ios)
      if ( ios .ne. 0 ) stop 'Para-file not found.'
c
      read (10,nml=les2)
      close(10)
c
      rsum = 1.0/dble((nx-2)*(ny-2))
c
      do m = 1,mf
        open (11,file=f_name(m)//r_name,form='unformatted',
     :           status='old',iostat=ios)
        if ( ios .ne. 0 ) cycle
c
        do it = 1,itplt
          read (11,end=100) dat
        end do
c
  100   close(11)
c
        do k = 1,nz
          ave = 0.0
        do j = 2,ny-1
        do i = 2,nx-1
          ave = ave +dat(i,k,j)
        end do
        end do
          vpro(k,m) = ave * rsum
        end do
c
      end do
c
c  #####
      do k = 2,6
        write(6,*) 'k, z*du/dz =', k,
     :       (k-1)*sqrt( ( vpro(k,1)-vpro(k-1,1) )**2
     :                  +( vpro(k,2)-vpro(k-1,2) )**2 )
      end do
c  #####
c
      wx   = 12.0
      wy   =  8.0
      shft =  0.2
c
      x0   =  4.5
      y0   =  7.0
c
c     kz   = nz
      kz   = 40
      zmax = dz * kz
      lbz  = nint( zmax/100.0 ) +400
c
cx    call winnam( 'Inflection' )
      call winfac( 2.0 )
      call plots ( 0.0, 0.0,  2 )
      call factor( 10.0 )
      call plot  (  x0,  y0, -3 )
      call newfnt( 4 )
c
      do m = 1,mt
        csm = -cos( th(m)*3.141593/180.0 )
        snm = -sin( th(m)*3.141593/180.0 )
c
        call newpen( 3 )
        call symbol(-1.50, wy*0.5, 0.50, z_lab,90.0,110 )
        call symbol( 0.5*wx,-1.20, 0.50, e_lab, 0.0,116 )
        call graph ( wx, wy, xsta, xend, 0.0, zmax, facx, facy,
     :         16, lbz,-1,-1, '      ', '      ', 0, 999.0, shft )
c
          call newpen( 0 )
          call plot  ( -xsta*facx, 0.0, 3 )
          call plot  ( -xsta*facx,  wy, 2 )
c
        do k = 1,kz
          tpro(k) = vpro(k,1)*csm + vpro(k,2)*snm
c
          xx(k) = ( tpro(k)-xsta )*facx
          yy(k) = ( k-0.5 )*dz    *facy +shft
        end do
c
        call smooth( tpro, kz )
        call smooth( tpro, kz )
c
          dif1 = tpro( 3 ) +tpro( 1 ) -2.0*tpro(2)
        do k = 3,kz-3
          dif2 = tpro(k+1) +tpro(k-1) -2.0*tpro(k)
c
          if ( dif1*dif2 .le. 0.0 ) then
            dp = abs( dif1/(dif2-dif1) )
            xp = ( tpro(k)*dp+tpro(k-1)*(1.0-dp)-xsta )*facx
            zp = ( k-1.5+dp )*dz                       *facy +shft
            call circle( xp, zp, 0.1, 0.1, 0.0, 360.0, 0 )
c?          exit
          end if
c
          dif1 = dif2
        end do
c
        call newpen( 4 )
        call polygn( xx, yy, kz-3, 0 )
        call number( xx(kz-3), yy(kz-3)+0.12, 0.4, th(m), 0.0, -101 )
        call symbol(   9999.9,        9999.9, 0.4, "$#6", 0.0,    3 )
c
      end do
c
      call cls  ( 1 )
      call plote
c
      stop
      end
c
      subroutine  smooth ( x, n )
c
      dimension  x(n), y(200)
c
      do i = 2,n-1
        y(i) = x(i) + 0.25*( x(i+1)+x(i-1)-2.0*x(i) )
      end do
c
      do i = 2,n-1
        x(i) = y(i)
      end do
c
      return
      end
