c
      subroutine  epestm3 (
     i                     iflag, u, v, w, t, fm, fh,
     i                     mx, mz, nx, nz, ny, dx, dz, dy, g, tref,
     w                     uu, vv, ww, uv, vw, wu, uav, vav,
     o                     epu, epv, epw )
c
c     Input variables:
c       iflag           : dummy; calculates eps
c
c       The apparent dissipation eps for the resolved-scale TKE is
c         eps = -<u_i'd(tau_ij')/dx_j> = -<u_i'd(tau_ij)/dx_j>
c             = -d<u_i'tau_ij>/dx_j +<tau_ij du_i'/dx_j>
c             = -d<u_i'tau_ij>/dx_j +<tau_ij du_i/dx_j> -<tau_ij>d<u_i>/dx_j
c             = - A + B - C,
c     where u_i and <u_i> are the resolved-scale and ensemble-average
c     velocity components, respectively, u_i = <u_i> + u_i', and
c     tau_ij = K_m (du_i/dx_j + du_j/dx_i).
c
      double precision  u (mx,mz,ny), v (mx,mz,ny), w (mx,mz,ny)
      double precision  t (mx,mz,ny), fm(mx,mz,ny), fh(mx,mz,ny)
      double precision  uu(mx,mz,ny), vv(mx,mz,ny), ww(mx,mz,ny),
     :                  uv(mx,mz,ny), vw(mx,mz,ny), wu(mx,mz,ny)
      double precision  uav(*), vav(*), av1, av2, av3, f1, f2
      dimension  epu(nz), epv(nz), epw(nz)
c
      anu  = 1.5e-5
      rsum = 1.0/dble((nx-2)*(ny-2))
c
      do k = 1,nz+1
        uav(k) = 0.0
        vav(k) = 0.0
        do j = 2,ny-1
        do i = 2,nx-1
          uav(k) = uav(k) +u(i,k,j)
          vav(k) = vav(k) +v(i,k,j)
        end do
        end do
        uav(k) = uav(k) * rsum
        vav(k) = vav(k) * rsum
      end do
c
c  **  Calculates eps  **
      do j = 1,ny-1
      do k = 1,nz
      do i = 1,nx-1
        f1 = 2.0 *( fm(i,k,j) +anu )
c
        uu(i,k,j) = f1*( u(i+1, k , j )-u(i,k,j) ) / dx
        vv(i,k,j) = f1*( v( i , k ,j+1)-v(i,k,j) ) / dy
        ww(i,k,j) = f1*( w( i ,k+1, j )-w(i,k,j) ) / dz
      end do
      end do
      end do
c
      do j = 2,ny
      do k = 1,nz
      do i = 2,nx
        f2 = 0.25*( fm( i ,k, j )+fm(i-1,k, j )
     :             +fm( i ,k,j-1)+fm(i-1,k,j-1) ) +anu
c
        uv(i,k,j) = f2*( ( u(i,k,j)-u( i ,k,j-1) ) / dy
     :                  +( v(i,k,j)-v(i-1,k, j ) ) / dx )
      end do
      end do
      end do
c
      do j = 2,ny
      do k = 2,nz+1
      do i = 2,nx
        f1 = 0.25*( fm(i, k ,j)+fm(i-1, k , j )
     :             +fm(i,k-1,j)+fm(i-1,k-1, j ) ) +anu
        f2 = 0.25*( fm(i, k ,j)+fm( i , k ,j-1)
     :             +fm(i,k-1,j)+fm( i ,k-1,j-1) ) +anu
c
        wu(i,k,j) = f1*( ( u(i,k,j)-u( i ,k-1, j ) ) / dz
     :                  +( w(i,k,j)-w(i-1, k , j ) ) / dx )
        vw(i,k,j) = f2*( ( v(i,k,j)-v( i ,k-1, j ) ) / dz
     :                  +( w(i,k,j)-w( i , k ,j-1) ) / dy )
      end do
      end do
      end do
c
      do k = 2,nz
        av1 = 0.0
        av2 = 0.0
        av3 = 0.0
c
        do j = 2,ny-1
        do i = 2,nx-1
          av1 = av1 +( u(i,k,j)-uav(k) )
     :            *( ( uu( i , k , j )-uu(i-1, k , j ) ) / dx
     :              +( uv( i , k ,j+1)-uv( i , k , j ) ) / dy
     :              +( wu( i ,k+1, j )-wu( i , k , j ) ) / dz )
          av2 = av2 +( v(i,k,j)-vav(k) )
     :            *( ( uv(i+1, k , j )-uv( i , k , j ) ) / dx
     :              +( vv( i , k , j )-vv( i , k ,j-1) ) / dy
     :              +( vw( i ,k+1, j )-vw( i , k , j ) ) / dz )
          av3 = av3 +0.5*w(i,k+1,j)
     :            *( ( wu(i+1,k+1, j )-wu( i ,k+1, j ) ) / dx
     :              +( vw( i ,k+1,j+1)-vw( i ,k+1, j ) ) / dy
     :              +( ww( i ,k+1, j )-ww( i , k , j ) ) / dz )
     :              +0.5*w(i, k ,j)
     :            *( ( wu(i+1, k , j )-wu( i , k , j ) ) / dx
     :              +( vw( i , k ,j+1)-vw( i , k , j ) ) / dy
     :              +( ww( i , k , j )-ww( i ,k-1, j ) ) / dz )
        end do
        end do
c
        epu(k) = av1 * rsum
        epv(k) = av2 * rsum
        epw(k) = av3 * rsum
      end do
c
      return
      end
