c
      subroutine  etestm (
     i                     iflag, t, fm, fh,
     i                     mx, mz, nx, nz, ny, dx, dz, dy,
     w                     uu, vv, ww, tav,
     o                     ept )
c
c     Input variables:
c       iflag           : = 0; calculates e_t
c                         <>0; calculates E_t
c
c       The dissipation <E_t> may be equivalent to the horizontal average of
c     the subgrid-scale dissipation e_t'. In this LES model, however, the
c     subgrid-scale motions are assumed to be in a state of local equilib-
c     rium (the diffusion is neglected), and <E_t> is estimated as follows.
c       The apparent dissipation e_t for the resolved-scale <t^2>/2 is
c         e_t = -<t'd(tau_tj')/dx_j> = -<t'd(tau_tj)/dx_j>
c             = -d<t'tau_tj>/dx_j +<tau_tj dt'/dx_j>
c             = -d<t'tau_tj>/dx_j +<tau_tj dt/dx_j> -<tau_tj>d<t>/dx_j
c             = - A + B - C,
c     where t and <t> are the resolved-scale and ensemble-average
c     temperature, respectively, t = <t> + t', and tau_tj = K_h dt/dx_j.
c       Since B and -C lead to the subgrid-scale production and a part
c     of the total one, the contribution of e_t to the real dissipation E_t
c     is -A, and E_t = -A +e_t'. Since e_t' = B in a state of local
c     equilibrium, E_t is
c         E_t = - A + B = e_t + C.
c
      double precision  t (mx,mz,ny), fm(mx,mz,ny), fh(mx,mz,ny)
      double precision  uu(mx,mz,ny), vv(mx,mz,ny), ww(mx,mz,ny)
      double precision  tav(*), ave, f1, f2, dtz
      dimension  ept(nz)
c
      tnu  = 2.1e-5
      rsum = 1.0/dble((nx-2)*(ny-2))
c
      do k = 1,nz+1
        tav(k) = 0.0
        do j = 2,ny-1
        do i = 2,nx-1
          tav(k) = tav(k) +t(i,k,j)
        end do
        end do
        tav(k) = tav(k) * rsum
      end do
c
c  **  Calculates e_t  **
      do j = 2,ny
      do k = 1,nz
      do i = 2,nx
        f1 = 0.5*( fh(i,k,j)+fh(i-1, k , j ) ) +tnu
        f2 = 0.5*( fh(i,k,j)+fh( i , k ,j-1) ) +tnu
c
        uu(i,k,j) = f1*( t(i,k,j)-t(i-1, k , j ) ) / dx
        vv(i,k,j) = f2*( t(i,k,j)-t( i , k ,j-1) ) / dy
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 2,nz+1
      do i = 2,nx-1
        f1 = 0.5*( fh(i,k,j)+fh( i ,k-1, j ) ) +tnu
c
        ww(i,k,j) = f1*( t(i,k,j)-t( i ,k-1, j ) ) / dz
      end do
      end do
      end do
c
      do k = 2,nz
        ave = 0.0
c
        do j = 2,ny-1
        do i = 2,nx-1
          ave = ave +( t(i,k,j)-tav(k) )
     :            *( ( uu(i+1, k , j )-uu( i , k , j ) ) / dx
     :              +( vv( i , k ,j+1)-vv( i , k , j ) ) / dy
     :              +( ww( i ,k+1, j )-ww( i , k , j ) ) / dz )
        end do
        end do
c
        ept(k) = ave * rsum
      end do
c
      if ( iflag .eq. 0 ) return
c
c  **  Calculates C  **
      do k = 2,nz+1
        dtz = ( tav(k)-tav(k-1) ) / dz
c
        do j = 2,ny-1
        do i = 2,nx-1
          uu(i,k,j) = ww(i,k,j)*dtz
        end do
        end do
      end do
c
      do k = 2,nz
        ave = 0.0
c
        do j = 2,ny-1
        do i = 2,nx-1
          ave = ave +0.5*( uu(i,k+1,j)+uu(i, k ,j) )
        end do
        end do
c
        ept(k) = ept(k) - ave*rsum
      end do
c
      return
      end
