c **********************************************************************
c *   Vertical profiles of temperature variance                        *
c *                                                                    *
c *                                   Dec/2005  M. Nakanishi (N.D.A)   *
c **********************************************************************
c
      include  "graphl.f"
      include  "sub/ziestm.f"
      include  "sub/t2estm.f"
      include  "sub/fsestm.f"
      include  "../src/Parameter.h"
c
      parameter ( mf=7, me=1, mt=5 )
      character (len=4)  :: f_name(mf), t_lab
      character (len=25) :: e_lab (me)
      character (len=3)  :: s_lab (me)
      character (len=10) :: z_lab
      character (len=5)  :: r_name
      double precision  dat(mx,mz,my,mf), ave, std
      double precision  wk (mx,mz,my, 4)
      dimension  vpro(mz,mt,me), hflx(mz)
      dimension  xsta(me), xend(me), lbx(me),  xx(mz), yy(mz)
      dimension  newp(mt), ihour(mt)
c
      data  f_name/ 't___', 'q___', 'u___', 'v___',
     :              'fm__', 'fh__', 'w___'/
      data  e_lab / '$<#q^2>/ T_*^2           '/
      data  s_lab /  '(a)'/
      data  z_lab / 'Height (m)'/, t_lab /'  00'/
      data  xsta  /  0.1/
      data  xend  /100.0/
      data  lbx   /   10/
      data  newp  /   4,  14,  54,  34,  44/
      data  ihour /   9,  10,  12,  14,  16/
c
      namelist  /les1/  nrun, nstep,  iout,  jout,  lout
      namelist  /les2/  isgs,  isfc,  irad,  icon,  isca,
     :                  jmin,  rlat,    dt,    dx,    dy,    dz,
     :                         ugsf,  vgsf,         aglw,  agmd,
     :                  gulw,  gvlw,  gumd,  gvmd,
     :                  tref,   tsf,   qsf,   psf,  hglw,  hgmd,
     :                  gtlw,  gqlw,  gtmd,  gqmd,  gamt,  gamq,
     :                   z0m,   z0h,  tfmx,  qfmx, phour, dhour,
     :                 tdeep,  swet,  crso,  fkso,
     :                  elat,  albd,  clov,  ulov,
     :                 nflay, frate,
     :                  nspl,  nspu,   csp, nstot
c
      write(6,*) 'Run + Restart number (xx.xx) =?'
      read (5,'(a)')  r_name
c
      open (10,file='para'//r_name,status='old',iostat=ios)
      if ( ios .ne. 0 ) stop 'Para-file not found.'
c
      read (10,nml=les1)
      read (10,nml=les2)
      close(10)
c
      do m = 1,mf
        open (m+10,file=f_name(m)//r_name,form='unformatted',
     :             status='old',iostat=ios)
        if ( ios .ne. 0 ) stop 'Data-file not found.'
      end do
c
      do m = 1,me
      do l = 1,mt
      do k = 1,mz
        vpro(k,l,m) = 0.0
      end do
      end do
      end do
c
      rsum = 1.0/dble((nx-2)*(ny-2))
c
      do ll = 1,999
        jhour = ( jmin +nint(ll*jout*dt)/60 )/60
c
        do m = 1,mf
          read (m+10,end=200) (((dat(i,k,j,m),i=1,mx),k=1,mz),j=1,my)
        end do
c
        do l = 1,mt
          if ( jhour .eq. ihour(l) ) goto 100
        end do
        cycle
c
  100   continue
c
        call fsestm( 1, dat(1,1,1,1), dat(1,1,1,7), dat(1,1,1,6),
     :               mx, mz, nx, nz, ny, dx, dz, dy,
     :               hflx )
c
        call t2estm( 1, dat(1,1,1,1), dat(1,1,1,5), dat(1,1,1,6),
     :               mx, mz, nx, nz, ny, dx, dz, dy,
     :               wk (1,1,1,1), wk (1,1,1,2), wk (1,1,1,3),
     :               wk (1,1,1,4),
     :               vpro(1,l,1) )
c
        ave = 0.0
        std = 0.0
        do j = 2,ny-1
        do i = 2,nx-1
          ave = ave +0.61*  dat(i,1,j,2) + 1.0
          std = std +0.61*( dat(i,1,j,1) + tref )
        end do
        end do
        ave = ave * rsum
        std = std * rsum
c
        if ( dhour .gt. 0.0 ) then
          chour = ihour(l)
          hcos  = cos( 1.570796*( chour-phour )/( 0.5*dhour ) )
          hflx1 = tfmx*hcos
          qflx1 = qfmx*hcos
        else
          hflx1 = tfmx
          qflx1 = qfmx
        end if
c
        hflx(1) = hflx1
        fpro    = hflx1*ave + qflx1*std
c
        call ziestm( hflx,nz, dz, 9.81,tref,ust, zi,wst,rate )
        if ( fpro .gt. 0.0 ) wst = ( 9.81/tref*fpro*zi )**( 1.0/3.0 )
        write(6,*) 'zi,wst,flx,erate =', zi, wst, hflx(1), rate
c
        tst = fpro/wst
c
        do k = 1,nz
          vpro(k,l, 1) = max( vpro(k,l,1)/tst**2, 0.5*xsta(1) )
        end do
c
      end do
c
  200 continue
c
      wx   = 7.4
      wy   = 8.0
      shft = 0.2
c
      xmv  = wx +1.6
      x0   = 5.5
      y0   = 7.0
c
      zmax = dz * nz
c     zdel = zmax/10.0
c     colm = 10.0**int( log10( zdel ) )
c     lbz  = zmax/( colm*int( zdel/colm ) )
      lbz  = 420
c
cx    call winnam( 'Profile' )
      call winfac( 2.5 )
      call plots ( 0.0, 0.0,  2 )
      call factor( 10.0 )
      call plot  (  x0,  y0, -3 )
      call newfnt( 4 )
c
      do m = 1,me
        xpls = 4.4
        ypls =-3.5
        ls   = 2
        zs   = 0.5
c
        call newpen( 3 )
        call symbol( 5.10, wy-1.0, 0.60, s_lab (m), 0.0,  3 )
        if ( mod(m,2) .eq. 1 )
     :  call symbol(-1.50, wy*0.5, 0.50, z_lab,    90.0,110 )
        call symbol( 0.5*wx,-1.20, 0.50, e_lab (m), 0.0,125 )
        call graphl( wx, wy, xsta(m), xend(m), 0.0, zmax, facx, facy,
     :               lbx(m),lbz,-1,-1, 0, shft )
c
        call rclip ( 1, 0.0, 0.0, wx, wy )
c
        do l = ls,mt
        ll = l -ls +1
          call newpen( newp(l) )
c
          if ( mod(m,2) .eq. 1 ) then
            write(t_lab(1:2),'(i2.2)') ihour(l)
            call plot  (0.6+xpls,wy-ll*0.5-1.3+ypls, 3)
            call plot  (1.3+xpls,wy-ll*0.5-1.3+ypls, 2)
            call symbol(1.5+xpls,wy-ll*0.5-1.3+ypls,0.45,t_lab,0.0,304)
          end if
c
          do k = 2,nz
            xx(k) = ( log10(vpro(k,l,m))-log10(xsta(m)) )*facx
            yy(k) = ( k-zs )*dz                          *facy +shft
          end do
c
          call polygn( xx(2), yy(2), nz-1, 0 )
c
        end do
c
        call rclip ( 0, 0.0, 0.0, wx, wy )
c
        if ( mod(m,2) .eq. 1 ) then
          call plot (  xmv,  0.0, -3 )
        else
          call plot ( -xmv,  0.0, -3 )
          call cls  ( 1 )
        end if
c
      end do
c
      call cls  ( 1 )
      call plote
c
      stop
      end
