c **********************************************************************
c *   Poisson equation for pressure (solved with SMAC method)          *
c *                                                                    *
c *   # Successive Over Relaxation (SOR) method is used to solve       *
c *     the linear system.                                             *
c **********************************************************************
c
      subroutine  poisson
c
      include  "Declare.h"
c
      pai = 3.141593
      tmp = 0.25*( cos( pai/real(nx-1) )+cos( pai/real(ny-1) ) )**2
c     omg = 2.0 /( 1.0+sqrt( 1.0-tmp ) )
      omg = 1.0
      eps = 1.0e-6 * max( ug(1,1)**2+vg(1,1)**2, 1.0 )
c
      rx2 = 1.0 / dx**2
      ry2 = 1.0 / dy**2
c
c  ***  Right-hand side term of the Poisson equation  ***
      do j = 2,ny-1
      do k = 1,nz
      do i = 1,mx
        uv(i,k,j) = -( u (i+1, k , j )-u (i,k,j) ) / dx
     :              -( v ( i , k ,j+1)-v (i,k,j) ) / dy
     :              -( w ( i ,k+1, j )-w (i,k,j) ) / dz
        uu(i,k,j) = 0.0
      end do
      end do
      end do
c
      do iter = 1,9999
c
        dpmx = 0.0
c
c  ***  The following can be omitted if c = 0.0 at k = 1. ***
cx      do j = 2,ny-1
cx      do i = 2,nx-1
cx        vv(i,1,j) = 1.0
cx        ww(i,1,j) = 0.0
cx      end do
cx      end do
c
!hpf$ independent
        do j = 2,ny-1
c
          do k = 1,nz
            km1 = max( k-1, 1 )
                            a  = 1.0 / dz**2
                            c  = a
            if ( k .eq. 1 ) c  = 0.0
          do i = 2,nx-1
            b  = a +c +2.0*( rx2+ry2 )
            d  = uu(i+1, k , j )*rx2 +uu(i-1, k , j )*rx2
     :          +uu( i , k ,j+1)*ry2 +uu( i , k ,j-1)*ry2
     :          +uv( i , k , j )
c
            den       =   b -c*vv(i,km1,j)
            vv(i,k,j) =   a                  / den
            ww(i,k,j) = ( d +c*ww(i,km1,j) ) / den
          end do
          end do
c
c  ***  Upper boundary condition  ***
          do i = 2,nx-1
            wu(i,nz+1,j) = ww(i,nz,j)/( 1.0-vv(i,nz,j) )
          end do
c
          do k = nz,1,-1
          do i = 2,nx-1
            wu(i,k,j) = vv(i,k,j)*wu(i,k+1,j) +ww(i,k,j)
            delp      = wu(i,k,j) -uu(i,k,j)
            uu(i,k,j) = uu(i,k,j) +omg*delp
c
            dpmx = max( dpmx, abs(delp) )
          end do
          end do
c
        end do
c
        call bc_lat_scalar ( uu )
cx      call bc_upp_scalar ( uu, 0.0 )
c
        if ( dpmx .lt. eps ) exit
      end do
c
      pav = 0.0
c
c     do j = 2,ny-1
c     do i = 2,nx-1
c       pav = pav +uu(i,nz,j)
c     end do
c     end do
c
c     pav = pav/dble((nx-2)*(ny-2))
c
c  ***  Correction of the pressure and wind vector  ***
      do j = 1,ny
      do k = 1,nz
      do i = 1,mx
        uu(i,k,j) = uu(i,k,j) -  pav
        pd(i,k,j) = pd(i,k,j) +  uu(i,k,j) / dt
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 1,nz
        m  = k +1
      do i = 2,nx-1
        u (i,k,j) = u (i,k,j) -( uu(i,k,j)-uu(i-1, k , j ) ) / dx
        v (i,k,j) = v (i,k,j) -( uu(i,k,j)-uu( i , k ,j-1) ) / dy
        w (i,m,j) = w (i,m,j) -( uu(i,m,j)-uu( i ,m-1, j ) ) / dz
      end do
      end do
      end do
c
      call bc_lat_moment
      call bc_upp_moment
c
      write(6,*) 'iter, omg, dpmx =', iter, omg, dpmx
c
      return
      end
