c **********************************************************************
c *   Making a file of the observational data                          *
c *                                                                    *
c *   Data stored in the file 'OBS':                                   *
c *     nzob          : Number of observational levels                 *
c *     tref          : Reference 'potential' temperature  (0.1deg.C)  *
c *     tsf           : Temperature at the ground surface  (0.1deg.C)  *
c *     psf           : Pressure    at the ground surface    (0.1hPa)  *
c *     hob           : Height above ground level                 (m)  *
c *     pob           : Pressure                             (0.1hPa)  *
c *     tob           : Temperature                        (0.1deg.C)  *
c *     qob           : Specific humidity                   (0.1g/kg)  *
c *     uob, vob      : Horizontal velocity components      (0.01m/s)  *
c *                                                                    *
c *   # All the data are desirable to be recorded in Integer type.     *
c **********************************************************************
c
      parameter ( mob=201 )
      dimension  hob(mob), pob(mob), tob(mob), qob(mob),
     :           uob(mob), vob(mob)
c
      open (12,file='KH.OBS')
c
      nzob = 151
      tref = 7.0
      psf  = 1000.0
c
      h    = 500.0/2.0
      zi   = h +500.0
      dz   = 10.0
      dt   =  0.125
      du   = -2.5
c
      do k = 1,nzob
        hob(k) = dz*( k-1 )
        pob(k) = 1000.0
c       tob(k) = dt*tanh( ( hob(k)-zi )/h ) +tref
        tob(k) = dt*      ( hob(k)-zi )/h   +tref
        qob(k) = 0.0
        uob(k) = du*tanh( ( hob(k)-zi )/h )
        vob(k) = 0.0
      end do
c
      tsf  = tob(1)
c
      write(12,'(5i5)')  nzob, nint(10.0*tref), nint(10.0*tsf),
     :                         nint(10.0*psf),  0
      write(12,'(51i5)'  )  ( nint(      hob(m)),m=1,nzob )
      write(12,'(51i5)'  )  ( nint( 10.0*pob(m)),m=1,nzob )
      write(12,'(51f5.3)')  (            tob(m) ,m=1,nzob )
      write(12,'(51i5)'  )  ( nint( 10.0*qob(m)),m=1,nzob )
      write(12,'(51i5)'  )  ( nint(100.0*uob(m)),m=1,nzob )
      write(12,'(51i5)'  )  ( nint(100.0*vob(m)),m=1,nzob )
c
      close(12)
c
      do k = nzob,1,-5
        write(6,*) 'hob,tob,uob=', hob(k), tob(k), uob(k)
      end do
c
      stop
      end
