c **********************************************************************
c *   Addition of fluctuations to flow fields below the nflay-th layer *
c *                                                                    *
c *   Input variables:                                                 *
c *     isfc          : < 0,       for sub.surface_stress_free         *
c *     nflay         : Number of layers to which fluctuations are     *
c *                   : added (Normally about nz/3)                    *
c *     frate         : Rate of fluctuation intensity to the geo-      *
c *                   : strophic wind (Normally about 5-6%)            *
c *     ugsf, vgsf    : Geostrophic wind                        (m/s)  *
c **********************************************************************
c
      subroutine  add_fluctuation ( isfc, nflay, frate, ugsf, vgsf )
c
      include  "Declare.h"
c
      if ( isfc .ge. 0 ) then
        kst = 1
        ked = min( nflay, nz/2 )
        zor = 0.0
      else
        ked = min( (nz+nflay)/2, nz )
        kst = nz -ked +1
        zor = 0.5*nz
      end if
c
      rt3  = 1.732051
      sigm = frate*max( sqrt( ugsf**2+vgsf**2 ), 1.0 )
c
      do j = 2,ny-1
        ir = j*2 -3
      do k = kst,ked
        m  = k +1
      do i = 2,nx-1
        fc1 = 1.0 -abs( k-0.5-zor )/( ked-zor )
        fc2 = 1.0 -abs( m-1.0-zor )/( ked-zor )
c
        tk(i,k,j) =            fc1*sigm**2
        u (i,k,j) = u (i,k,j) +fc1*sigm *rt3*urand( ir )
        v (i,k,j) = v (i,k,j) +fc1*sigm *rt3*urand( ir )
        w (i,m,j) = w (i,m,j) +fc2*sigm *rt3*urand( ir )
c
c?      t (i,k,j) = t (i,k,j) +fc1*5.e-2*rt3*urand( ir )
c?      q (i,k,j) = q (i,k,j) +fc1*5.e-6*rt3*urand( ir )
      end do
      end do
      end do
c
      call bc_lat_moment
cx    call bc_lat_scalar ( tk )
c
c?    call bc_lat_scalar ( t  )
c?    call bc_lat_scalar ( q  )
c
      call poisson
c
      do j = 1,ny
      do k = 1,mz
      do i = 1,mx
        pd(i,k,j) = 0.0
      end do
      end do
      end do
c
      return
      end
c
      function  urand ( ix )
c
      integer*8  ia, ib, im
c
      ia  =      32771
      ib  = 1234567891
      im  = 2147483648.5d0
c
      ix  = mod( ia*ix + ib, im )
c
      urand = 2.0d0*( dble(ix)/dble(im)-0.5d0 )
c
      return
      end
