c **********************************************************************
c *   Input of observational vertical distributions                    *
c *                                                                    *
c *   Data stored in the file 'OBS':                                   *
c *     nzob          : Number of observational levels                 *
c *     tref          : Reference 'potential' temperature  (0.1deg.C)  *
c *     tsf           : Temperature at the ground surface  (0.1deg.C)  *
c *     psf           : Pressure    at the ground surface    (0.1hPa)  *
c *     hob           : Height above ground level                 (m)  *
c *     pob           : Pressure                             (0.1hPa)  *
c *     tob           : Temperature                        (0.1deg.C)  *
c *     qob           : Specific humidity                   (0.1g/kg)  *
c *     uob, vob      : Horizontal velocity components      (0.01m/s)  *
c *                                                                    *
c *   # Units of data are applied to the Integer type in ASCII file.   *
c **********************************************************************
c
      subroutine  initial_input ( tsf, psf )
c
      include  "Declare.h"
c
      parameter ( mob=201 )
      dimension  hob(mob), pob(mob), tob(mob), qob(mob),
     :           uob(mob), vob(mob)
c
      write(6,*) 'sub.initial_input: start'
c
c  ***  For ASCII file  ***
      open (12,file='OBS',status='old')
c
      read (12,'(i5,3f5.1)',err=10)  nzob, tref, tsf, psf
c
      if ( mob .lt. nzob ) goto 999
c
      read (12,'(51f5.0)')  ( hob(m),m=1,nzob )
      read (12,'(51f5.1)')  ( pob(m),m=1,nzob )
      read (12,'(51f5.1)')  ( tob(m),m=1,nzob )
      read (12,'(51f5.1)')  ( qob(m),m=1,nzob )
      read (12,'(51f5.2)')  ( uob(m),m=1,nzob )
      read (12,'(51f5.2)')  ( vob(m),m=1,nzob )
c
      goto 20
c
   10 close(12)
c
c  ***  For BINARY file  ***
      open (12,file='OBS',status='old',form='unformatted')
c
      read (12)  nzob, tref, tsf, psf
c
      if ( mob .lt. nzob ) goto 999
c
      read (12)  ( hob(m),m=1,nzob )
      read (12)  ( pob(m),m=1,nzob )
      read (12)  ( tob(m),m=1,nzob )
      read (12)  ( qob(m),m=1,nzob )
      read (12)  ( uob(m),m=1,nzob )
      read (12)  ( vob(m),m=1,nzob )
c
   20 continue
c
        tref   = nint( ( tref+t0 )*100.0 ) / 100.0
        tsf    = ( tsf   +t0 )*( p0/psf    )**( rd/cp )
c
      do m = 1,nzob
        tob(m) = ( tob(m)+t0 )*( p0/pob(m) )**( rd/cp ) - tref
        qob(m) =   qob(m)     *  1.0e-3
      end do
c
      do k = 1,nz
        zk = dz*( k-0.5 )
c
        do m = 2,nzob
          if ( hob(m) .ge. zk ) goto 100
        end do
c
        write(6,*) 'Observational top level,', hob(nzob), ', is low.'
        stop
c
  100   delz = ( hob(m)-zk )/( hob(m)-hob(m-1) )
c
        ui(k) = uob(m)*(1.0-delz) + uob(m-1)*delz
        vi(k) = vob(m)*(1.0-delz) + vob(m-1)*delz
        ti(k) = tob(m)*(1.0-delz) + tob(m-1)*delz
        qi(k) = qob(m)*(1.0-delz) + qob(m-1)*delz
      end do
c
      do k = nz+1,mz
        ui(k) = ui(nz)
        vi(k) = vi(nz)
        ti(k) = ti(nz) + gamt*dz*( k-nz )
        qi(k) = qi(nz) + gamq*dz*( k-nz )
      end do
c
      close(12)
c
      return
c
  999 stop 'sub.initial_input: mob must be larger than nzob.'
      end
