c **********************************************************************
c *   Poisson equation for pressure (solved with SMAC method)          *
c *                                                                    *
c *   # The subroutine of FFT is not parallelized.                     *
c **********************************************************************
c
      subroutine  poisson
c
      include  "Declare.h"
c
      dimension  trix(mx), triy(my), ifax(20), ifay(20)
      dimension  wno(mx,my)
!hpf$ distribute (*,block) :: wno
c
      data  new / 0 /
c
      save  new, trix, triy, ifax, ifay, wno
c
      interface
        subroutine fft_fb ( uu, trix, triy, ifax, ifay, ifb )
          include  "Implicit.h"
          include  "Parameter.h"
c
          dimension  uu(mx,mz,my)
          dimension  trix(mx), triy(my), ifax(20), ifay(20)
          intent (in)     ifb
          intent (inout)  uu, trix, triy, ifax, ifay
!hpf$     distribute (*,block,*) :: uu
        end subroutine
      end interface
c
      if ( new .ne. 0 ) goto 100
c
        new = 1
c
        call rfftim ( nx-2, trix, ifax )
        call rfftim ( ny-2, triy, ifay )
c
        do j = 2,ny-1
        do i = 2,nx-1
          ix = ( i-1 ) / 2
          iy = ( j-1 ) / 2
          xx = sin( 3.14159265358979d0*dble(ix)/dble(nx-2) ) / dx
          yy = sin( 3.14159265358979d0*dble(iy)/dble(ny-2) ) / dy
c
c  ***  wno is the square of the modified wavenumber (m^[-2])  ***
c  ***  according to the staggered grid arrangement.           ***
          wno(i,j) = 4.0*( xx**2 +yy**2 )
        end do
        end do
c
  100 continue
c
c  ***  Right-hand side term of the Poisson equation  ***
      do j = 2,ny-1
      do k = 1,nz
      do i = 1,mx
        uu(i,k,j) = -( u (i+1, k , j )-u (i,k,j) ) / dx
     :              -( v ( i , k ,j+1)-v (i,k,j) ) / dy
     :              -( w ( i ,k+1, j )-w (i,k,j) ) / dz
      end do
      end do
      end do
c
c  ***  Forward Fourier transform  ***
      call fft_fb ( uu, trix, triy, ifax, ifay,  1 )
c
c  ***  The following can be omitted if c = 0.0 at k = 1. ***
cx    do j = 2,ny-1
cx    do i = 2,nx-1
cx      vv(i,1,j) = 1.0
cx      ww(i,1,j) = 0.0
cx    end do
cx    end do
c
!hpf$ independent
      do j = 2,ny-1
      do k = 1,nz
        km1 = max( k-1, 1 )
                        a  = 1.0 / dz**2
                        c  = a
        if ( k .eq. 1 ) c  = 0.0
      do i = 2,nx-1
        b  = a +c +wno(i,j)
        d  = uu(i,k,j)
c
        den       =   b -c*vv(i,km1,j)
        vv(i,k,j) =   a                  / den
        ww(i,k,j) = ( d +c*ww(i,km1,j) ) / den
      end do
      end do
      end do
c
c  ***  The following set the horizontal average of pd  ***
c  ***  at the top boundary to zero.                    ***
      vv(2,nz,2) = 0.0
      ww(2,nz,2) = 0.0
c
c  ***  Upper boundary condition  ***
      do j = 2,ny-1
      do i = 2,nx-1
        uu(i,nz+1,j) = ww(i,nz,j)/( 1.0-vv(i,nz,j) )
      end do
      end do
c
      do j = 2,ny-1
      do k = nz,1,-1
      do i = 2,nx-1
        uu(i,k,j) = vv(i,k,j)*uu(i,k+1,j) +ww(i,k,j)
      end do
      end do
      end do
c
c  ***  Backward Fourier transform  ***
      call fft_fb ( uu, trix, triy, ifax, ifay, -1 )
c
      call bc_lat_scalar ( uu )
cx    call bc_upp_scalar ( uu, 0.0 )
c
c  ***  Correction of the pressure and wind vector  ***
      do j = 1,ny
      do k = 1,nz
      do i = 1,mx
        pd(i,k,j) = pd(i,k,j) +  uu(i,k,j) / dt
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 1,nz
        m  = k +1
      do i = 2,nx-1
        u (i,k,j) = u (i,k,j) -( uu(i,k,j)-uu(i-1, k , j ) ) / dx
        v (i,k,j) = v (i,k,j) -( uu(i,k,j)-uu( i , k ,j-1) ) / dy
        w (i,m,j) = w (i,m,j) -( uu(i,m,j)-uu( i ,m-1, j ) ) / dz
      end do
      end do
      end do
c
      call bc_lat_moment
      call bc_upp_moment
c
      return
      end
c
      subroutine  fft_fb ( uu, trix, triy, ifax, ifay, ifb )
c
      include  "Implicit.h"
      include  "Parameter.h"
c
      dimension  uu(mx,mz,my), wk1(mx,my), wk2(mx,my)
      dimension  trix(mx), triy(my), ifax(20), ifay(20)
      intent (in)     ifb
      intent (inout)  uu, trix, triy, ifax, ifay
!hpf$ distribute (*,block,*) :: uu
c
      interface
        pure subroutine rfftfm
     :         ( n, inc, jump, lot, r, wa, ifac, wsave )
          include  "Implicit.h"
          dimension  r(*), wsave(*), wa(n), ifac(*)
          intent (in)     n, inc, jump, lot
          intent (inout)  r, wsave, wa, ifac
        end subroutine
c
        pure subroutine rfftbm
     :         ( n, inc, jump, lot, r, wa, ifac, wsave )
          include  "Implicit.h"
          dimension  r(*), wsave(*), wa(n), ifac(*)
          intent (in)     n, inc, jump, lot
          intent (inout)  r, wsave, wa, ifac
        end subroutine
      end interface
c
      nx2 = nx -2
      ny2 = ny -2
c
!hpf$ independent, new(i, j, wk1, wk2, trix, triy, ifax, ifay)
      do k = 1,nz
        do j = 1,ny2
        do i = 1,nx2
          wk1(i,j) = uu(i+1, k ,j+1)
        end do
        end do
c
        if ( ifb .gt. 0 ) then
          call rfftfm ( nx2, 1,mx,ny2, wk1, trix,ifax, wk2 )
          call rfftfm ( ny2,mx, 1,nx2, wk1, triy,ifay, wk2 )
        else
          call rfftbm ( nx2, 1,mx,ny2, wk1, trix,ifax, wk2 )
          call rfftbm ( ny2,mx, 1,nx2, wk1, triy,ifay, wk2 )
        end if
c
        do j = 1,ny2
        do i = 1,nx2
          uu(i+1, k ,j+1) = wk1(i,j)
        end do
        end do
      end do
c
      return
      end
