c **********************************************************************
c *   Time integration for momentum                                    *
c *                                                                    *
c *   Input variables:                                                 *
c *     a             : Coefficient for time integration               *
c *                   : = 1.0,     Euler forward scheme                *
c *                   : = 1.5,     Adams-Bashforth scheme              *
c *     pmx, pmy      : Mean forcing                          (m/s^2)  *
c **********************************************************************
c
      subroutine  predict_moment ( a, pmx, pmy )
c
      include  "Declare.h"
c
      dimension  w2(mx,mz,my)
!hpf$ distribute (*,*,block) :: w2
c
c  #####
      rv  = 1.0/100000.0
      fc  = 0.0
c  #####
      aa  = a
      bb  = 1.0 -aa
c
      if ( fs .eq. 0.0 ) then
        pax = pmx
        pay = pmy
      else
        pax = 0.0
        pay = 0.0
      end if
c
      do j = 1,ny
      do k = 1,nz
      do i = 1,mx
        wo(i,k,j) = u (i,k,j)
        w2(i,k,j) = v (i,k,j)
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 1,nz
      do i = 2,nx-1
        fu = ( uu( i , k , j )-uu(i-1, k , j ) ) / dx
     :      +( uv( i , k ,j+1)-uv( i , k , j ) ) / dy
     :      +( wu( i ,k+1, j )-wu( i , k , j ) ) / dz
     :      +fs*( 0.25*( v ( i , k ,j+1)+v (i-1, k ,j+1)
     :                  +v ( i , k , j )+v (i-1, k , j ) )-vg(k) )
     :      -fc*  0.25*( w ( i ,k+1, j )+w (i-1,k+1, j )
     :                  +w ( i , k , j )+w (i-1, k , j ) )
     :   +rv*( 0.125*( ( v ( i , k ,j+1)+v ( i , k , j ) )**2
     :                +( v (i-1, k ,j+1)+v (i-1, k , j ) )**2 )
     :                                                    -vg(k)**2 )
c
        u  (i,k,j) = u (i,k,j) +dt*( aa*fu +bb*fub(i,k,j) +pax
     :                       -( pd(i,k,j)-pd(i-1, k , j ) )/dx )
        fub(i,k,j) = fu
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 1,nz
      do i = 2,nx-1
        fv = ( uv(i+1, k , j )-uv( i , k , j ) ) / dx
     :      +( vv( i , k , j )-vv( i , k ,j-1) ) / dy
     :      +( vw( i ,k+1, j )-vw( i , k , j ) ) / dz
     :      -fs*( 0.25*( wo(i+1, k , j )+wo(i+1, k ,j-1)
     :                  +wo( i , k , j )+wo( i , k ,j-1) )-ug(k) )
     :   -rv*( 0.125*( ( wo(i+1, k , j )+wo( i , k , j ) )
     :                *( w2( i , k ,j+1)+w2( i , k , j ) )
     :                +( wo(i+1, k ,j-1)+wo( i , k ,j-1) )
     :                *( w2( i , k , j )+w2( i , k ,j-1) ) )
     :                                                    -ug(k)*vg(k) )
c
        v  (i,k,j) = v (i,k,j) +dt*( aa*fv +bb*fvb(i,k,j) +pay
     :                       -( pd(i,k,j)-pd( i , k ,j-1) )/dy )
        fvb(i,k,j) = fv
      end do
      end do
      end do
c
      do j = 2,ny-1
      do k = 2,nz
      do i = 1,mx
        fw = ( wu(i+1, k , j )-wu( i , k , j ) ) / dx
     :      +( vw( i , k ,j+1)-vw( i , k , j ) ) / dy
     :      +( ww( i , k , j )-ww( i ,k-1, j ) ) / dz
     :      +( bv( i , k , j )+bv( i ,k-1, j ) ) * 0.5
     :      +fc*  0.25*( wo(i+1, k , j )+wo(i+1,k-1, j )
     :                  +wo( i , k , j )+wo( i ,k-1, j ) )
c
        w  (i,k,j) = w (i,k,j) +dt*( aa*fw +bb*fwb(i,k,j)
     :                       -( pd(i,k,j)-pd( i ,k-1, j ) )/dz )
        fwb(i,k,j) = fw
      end do
      end do
      end do
c
c  ***  The following will be carried out in sub.sponge_layer  ***
cx    call bc_lat_moment
cx    call bc_upp_moment
c
      return
      end
